from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "MarginTradingPlateMapping",
    "MarginTradingPlate",
]


class MarginTradingPlateMapping(BaseMapping):
    """字段映射 融资融券账户统计-板块融资融券"""
    columns: Dict = {
        "BOARD_CODE": "板块代码",
        "BOARD_NAME": "板块名称",
        "BOARD_TYPE": "板块类型",
        "BOARD_TYPE_CODE": "板块类型代码",
        "END_DATE": "结束日期",
        "FIN_BALANCE": "融资余额",
        "MARGIN_BALANCE": "保证金余额",
        "FIN_BALANCE_DIFF": "融资余额差额",
        "LOAN_NETSELL_AMT": "借入净卖出金额",
        "LOAN_BALANCE": "借入余额",
        "NOTLIMITED_MARKETCAP_A": "非限制市场总值A",
        "FIN_BALANCE_RATIO": "融资余额比例",
        "INTERVAL_TYPE": "区间类型",
        "FIN_BUY_AMT": "融资买入金额",
        "FIN_REPAY_AMT": "融资偿还金额",
        "LOAN_BALANCE_VOL": "借入余额量",
        "FIN_NETSELL_AMT": "融资净卖出金额",
        "LOAN_SELL_VOL": "借入卖出量",
        "LOAN_REPAY_VOL": "借入偿还量",
        "LOAN_SELL_AMT": "借入卖出金额",
        "FIN_NETBUY_AMT": "融资净买入金额"
    }


class MarginTradingPlate(APIDataV1RequestData):
    """查询 融资融券账户统计-板块融资融券"""
    def __init__(
            self,
            plate_name: Literal["行业", "概念", "地域"] = "行业",
            cycle: Literal[1, 3, 5, 10] = 1,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.margin import *

            # 查询 融资融券账户统计-板块融资融券
            data = MarginTradingPlate(plate_name="行业").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            plate_name: 板块类型: `["行业", "概念", "地域"]`
            cycle: 统计周期
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.plate_name = plate_name
        self.cycle = cycle
        self.size = size
        self.mapping = MarginTradingPlateMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="融资融券账户统计-板块融资融券")
        self.conditions = []

    def params_report_name(self) -> str:
        """"""
        if self.cycle == 1:
            return "RPTA_WEB_BKJYMXN"
        else:
            return "RPTA_WEB_BKQJYMXN"

    def params_plate_code(self) -> str:
        """"""
        plate_mapping = {"行业": "005", "概念": "006", "地域": "004"}
        return plate_mapping[self.plate_name]

    def params_filter(self) -> str:
        """"""
        self.conditions.append(f'(BOARD_TYPE_CODE="{self.params_plate_code()}")')
        if self.cycle > 1:
            self.conditions.append(f'(INTERVAL_TYPE="{self.cycle}日")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": self.params_report_name(),
            "sortColumns": "FIN_NETBUY_AMT",
            "sortTypes": "-1",
            "pageSize": self.size,
            "stat": self.cycle,
            "filter": self.params_filter(),
            "pageNo": "1",
        }
        return self.base_param(update=params)



from typing import Dict
from pydantic import BaseModel


__all__ = [
    "headers"
]


class Headers(BaseModel):
    """"""
    headers: Dict = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36',
    }


headers = Headers()




import unittest
from wisecon.fund import FundList


class TestFundList(unittest.TestCase):
    def test_fund_list(self):
        data = FundList().load()
        data.show_columns()
        print(data.to_frame().head().to_markdown())
        print(data.to_frame(chinese_column=True).tail().to_markdown())



from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "HoseIndexOldMapping",
    "HoseIndexOld",
    "HoseIndexNewMapping",
    "HoseIndexNew",
]


class HoseIndexOldMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "HOSE_INDEX": "国房景气指数（指数值）",
        "HOSE_INDEX_SAME": "国房景气指数（同比增长）",
        "LAND_INDEX": "土地开发面积指数（指数值）",
        "LAND_INDEX_SAME": "土地开发面积指数（指数值）",
        "GOODSHOSE_INDEX": "销售价格指数（同比增长）",
        "GOODSHOSE_INDEX_SAME": "销售价格指数（同比增长）",
    }


class HoseIndexOld(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = HoseIndexOldMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 房价指数(08—10年)")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "HOSE_INDEX", "HOSE_INDEX_SAME", "LAND_INDEX",
            "LAND_INDEX_SAME", "GOODSHOSE_INDEX", "GOODSHOSE_INDEX_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_HOSE_INDEX",
        }
        return self.base_param(update=params)


class HoseIndexNewMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "时间",
        "CITY": "城市",
        "FIRST_COMHOUSE_SAME": "新建商品住宅价格指数（同比）",
        "FIRST_COMHOUSE_SEQUENTIAL": "新建商品住宅价格指数（环比）",
        "FIRST_COMHOUSE_BASE": "新建商品住宅价格指数（定基）",
        "SECOND_HOUSE_SAME": "二手住宅价格指数（同比）",
        "SECOND_HOUSE_SEQUENTIAL": "二手住宅价格指数（环比）",
        "SECOND_HOUSE_BASE": "二手住宅价格指数（定基）",
        "REPORT_DAY": "日期",
    }


class HoseIndexNew(MacroRequestData):
    """"""
    def __init__(
            self,
            cities: Optional[List[str]] = ["北京", "上海"],
            report_date: Optional[str] = None,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """

        :param query_config:
        :param cities: 城市
        :param size: 数据量
        :param report_date: 日期
        :param verbose:
        :param logger:
        :param kwargs:
        """
        self.cities = cities
        self.report_date = report_date
        self.size = size
        self.mapping = HoseIndexNewMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 新房价指数")

    def _data_filter(self):
        """"""
        if self.report_date:
            _filter = f"(REPORT_DATE='{self.report_date}')"
        elif self.cities:
            cities = [f'"{item}"' for item in self.cities]
            _filter = f'(CITY+in+({",".join(cities)}))'
        else:
            raise ValueError("report_date or cities must be set")
        return _filter

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "CITY", "FIRST_COMHOUSE_SAME", "FIRST_COMHOUSE_SEQUENTIAL",
            "FIRST_COMHOUSE_BASE", "SECOND_HOUSE_SAME", "SECOND_HOUSE_SEQUENTIAL",
            "SECOND_HOUSE_BASE", "REPORT_DAY"
        ]

        params = {
            "columns": ",".join(columns),
            "filter": self._data_filter(),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE,CITY",
            "sortTypes": "-1,-1",
            "reportName": "RPT_ECONOMY_HOUSE_PRICE",
        }
        return params



import unittest
from wisecon.utils.time import *


class TestTime(unittest.TestCase):
    def test_year_to_start_end(self):
        """"""
        start_date, end_date = year2date(2022, format="%Y-%m-%d")
        print(start_date, type(start_date), end_date, type(end_date))



import json
from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseRequestData, BaseMapping
from wisecon.utils import filter_str_by_mark


__all__ = [
    "FundValueMapping",
    "FundValue",
]


class FundValueMapping(BaseMapping):
    """字段映射 基金当前净值"""
    columns: Dict = {
        "fundcode": "基金代码",
        "name": "基金名称",
        "jzrqv": "上一交易日",
        "dwjz": "基金净值（截止上一交易日）",
        "gsz": "估算净值（实时）",
        "gszzl": "估算涨幅（实时）",
        "gztime": "更新时间（实时）",
    }


class FundValue(BaseRequestData):
    """查询 基金当前净值"""
    def __init__(
            self,
            fund_code: str,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.fund.fund_value import FundValue

            data = FundValue(fund_code="000001").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            fund_code: 基金代码
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.fund_code = fund_code
        self.mapping = FundValueMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

        self.request_set(
            response_type="text",
            description="基金当前净值",
        )

    def base_url(self, ) -> str:
        """"""
        base_url = f"http://fundgz.1234567.com.cn/js/{self.fund_code}.js"
        return base_url

    def clean_content(
            self,
            content: Optional[str] = None,
    ) -> List[Dict]:
        """"""
        content = filter_str_by_mark(content)
        data = [json.loads(content)]
        return data



import time
from datetime import datetime
from typing import Tuple, Literal


__all__ = [
    "time2int",
    "year2date",
]


def time2int() -> str:
    """将当前时间转换为毫秒级时间戳

    Returns:
        毫秒级时间戳
    """
    return str(int(time.time() * 1E3))


def year2date(
        year: int,
        format: Literal["%Y%m%d", "%Y-%m-%d"] = "%Y%m%d"
) -> Tuple[str, str]:
    """给定一个年份，返回该年份的开始与结束日期

    Args:
        year: 年份
        format: 日期格式

    Returns:
        start_date: 开始日期
        end_date: 结束日期
    """
    start_date = datetime(year, 1, 1).strftime(format)
    end_date = datetime(year, 12, 31).strftime(format)
    return start_date, end_date



from typing import List, Dict, Optional
from wisecon.types.request_data import BaseRequestData


__all__ = [
    "APIMovieRequestData",
    "APITVRequestData",
]


class APIMovieRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://piaofang.maoyan.com/getBoxList"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {"date": 1, "isSplit": "true"}
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        tv_list = json_data.get("boxOffice", {})
        data = tv_list.pop("data")
        self.metadata.response = data.get("updateInfo")
        data_list = data.get("list")
        for item in data_list:
            movie_item = item.pop("movieInfo")
            item.update(movie_item)
        return data_list


class APITVRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://piaofang.maoyan.com/getTVList"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {"showDate": 2}
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        tv_list = json_data.get("tvList", {})
        data = tv_list.pop("data")
        self.metadata.response = data.get("updateInfo")
        return data.get("list")



from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIMainHolderDetail, BaseMapping


__all__ = [
    "StockHolderMapping",
    "StockHolder",
]


class StockHolderMapping(BaseMapping):
    """字段映射 股票的机构持有者清单"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "SECURITY_INNER_CODE": "证券内部代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "REPORT_DATE": "报告日期",
        "HOLDER_CODE": "持有人代码",
        "HOLDER_NAME": "持有人名称",
        "PARENT_ORG_CODE": "母组织代码",
        "PARENT_ORGCODE_OLD": "旧母组织代码",
        "PARENT_ORG_NAME": "母组织名称",
        "ORG_TYPE_CODE": "组织类型代码",
        "ORG_TYPE": "组织类型",
        "TOTAL_SHARES": "总持股数",
        "HOLD_MARKET_CAP": "持有市值",
        "TOTAL_SHARES_RATIO": "总持股比例",
        "FREE_SHARES_RATIO": "自由流通股比例",
        "NETASSET_RATIO": "净资产比例",
        "ORG_NAME_ABBR": "组织名称简称",
        "BuyState": "购买状态"
    }


class StockHolder(APIMainHolderDetail):
    """查询 股票的机构持有者清单"""

    def __init__(
            self,
            security_code: Optional[str] = None,
            holder: Optional[Literal["基金", "QFII", "社保", "券商", "保险", "信托"]] = None,
            date: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.main_holder import *

            data = StockHolder(security_code="603350", holder="基金", date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 股票代码
            holder: 机构类型
            date: 查询日期
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.holder = holder
        self.date = date
        self.size = size
        self.mapping = StockHolderMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="股票的机构持有者清单",)
        self.validate_security_code(code=security_code)
        self.validate_date_is_end_if_quarter(date=date)

    def params_holder_type(self) -> int:
        """"""
        holder_mapping = {"基金": 1, "QFII": 2, "社保": 3, "券商": 4, "保险": 5, "信托": 6}
        if self.holder in holder_mapping:
            return holder_mapping[self.holder]
        else:
            raise ValueError(f"holder_code must be in {holder_mapping.keys()}")

    def params(self) -> Dict:
        """"""
        params = {
            "ReportDate": self.date,
            "SHType": self.params_holder_type(),
            "SCode": self.security_code,
            "sortField": "HOLDER_CODE",
            "pageSize": self.size,
        }
        return self.base_param(update=params)



import json
from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseMapping, BaseRequestData
from wisecon.utils import filter_str_by_mark


__all__ = [
    "FundListMapping",
    "FundList",
]


class FundListMapping(BaseMapping):
    """字段映射 全部基金列表"""
    columns: Dict = {
        "fund_code": "基金代码",
        "fund_pinyin": "基金拼音简写",
        "fund_name": "基金名称",
        "fund_type": "基金类型",
        "fund_pinyin_full": "基金拼音全称"
    }


class FundList(BaseRequestData):
    """查询 全部基金列表"""
    def __init__(
            self,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.fund import *

            data = FundList().load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.mapping = FundListMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="text",
            description="基金列表",
        )

    def base_url(self) -> str:
        """"""
        base_url = "http://fund.eastmoney.com/js/fundcode_search.js"
        return base_url

    def clean_content(self, content: Optional[str]) -> List[Dict]:
        """"""
        content = filter_str_by_mark(s=content, start="[", end="]")
        data = json.loads(content)

        columns = list(self.mapping.columns.keys())

        def _clean_data(item):
            """"""
            return dict(zip(columns, item))

        data = list(map(_clean_data, data))
        return data



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "TransferFundMapping",
    "TransferFund",
]


class TransferFundMapping(BaseMapping):
    """"""
    columns: Dict = {
        "START_DATE": "开始时间",
        "END_DATE": "截至时间",
        "END_SETTLE_BALANCE": "交易结算资金期末余额(亿)",
        "AVG_SETTLE_BALANCE": "交易结算资金日均余额(亿)",
        "SETTLE_FUNDS_ADD": "银证转账增加额(亿)",
        "SETTLE_FUNDS_REDUCE": "银证转账减少额(亿)",
        "SETTLE_FUNDS_NET": "银证转账变动净额(亿)",
        # "END_SETTLE_BALANCE_QOQ": "",
        # "AVG_SETTLE_BALANCE_QOQ": "",
        "INDEX_PRICE_SH": "上证指数收盘",
        "INDEX_PRICE_SZ": "深证指数收盘",
        # "INDEX_PRICE_CY": "",
        # "INDEX_PRICE_ZX": "",
        "INDEX_CHANGE_RATIO_SH": "上证指数涨跌幅",
        "INDEX_CHANGE_RATIO_SZ": "深证指数涨跌幅",
        # "INDEX_CHANGE_RATIO_CY": "",
        # "INDEX_CHANGE_RATIO_ZX": "",
    }


class TransferFund(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = TransferFundMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 交易结算资金(银证转账) 据投保基金公司2017年7月31日公告，该数据已停止更新。")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "START_DATE", "END_DATE", "END_SETTLE_BALANCE", "AVG_SETTLE_BALANCE", "SETTLE_FUNDS_ADD",
            "SETTLE_FUNDS_REDUCE", "SETTLE_FUNDS_NET", "END_SETTLE_BALANCE_QOQ", "AVG_SETTLE_BALANCE_QOQ",
            "INDEX_PRICE_SH", "INDEX_PRICE_SZ", "INDEX_PRICE_CY", "INDEX_PRICE_ZX", "INDEX_CHANGE_RATIO_SH",
            "INDEX_CHANGE_RATIO_SZ", "INDEX_CHANGE_RATIO_CY", "INDEX_CHANGE_RATIO_ZX"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "END_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_BANKSECURITY_TRANSFER_FUND",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIAnalystInvest


__all__ = [
    "AnalystYearRankMapping",
    "AnalystYearRank",
]


class AnalystYearRankMapping(BaseMapping):
    """字段映射 分析师指数-年度排行榜"""
    columns: Dict = {
        "ANALYST_CODE": "分析师代码",
        "ANALYST_NAME": "分析师姓名",
        "TRADE_DATE": "交易日期",
        "YEAR": "年份",
        "ORG_NAME": "机构名称",
        "ORG_CODE": "机构代码",
        "INDEX_VALUE": "指数值",
        "YEAR_YIELD": "年度收益率",
        "YIELD_3": "3个月收益率",
        "YIELD_6": "6个月收益率",
        "YIELD_12": "12个月收益率",
        "SECURITY_COUNT": "证券数量",
        "SECURITY_NAME_ABBR": "证券简称",
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "NEWEST_STOCK_RATING": "最新股票评级",
        "INDUSTRY_CODE": "行业代码",
        "INDUSTRY_NAME": "行业名称"
    }


class AnalystYearRank(APIAnalystInvest):
    """查询 分析师指数-年度排行榜"""
    def __init__(
            self,
            year: Optional[str] = None,
            top_n: Optional[int] = None,
            industry_code: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.analyst import *
            from wisecon.stock.plate_mapping import *

            # 查询全部 2024 年度排行榜
            data = AnalystYearRank(year="2024").load()
            print(data.to_markdown(chinese_column=True))

            # 指定行业编码
            data = AnalystYearRank(year="2024", industry_code="480000").load()
            print(data.to_markdown(chinese_column=True))

            # 查询行业编码
            data = IndustryCode().load()
            print(data.to_markdown(chinese_column=True))
            ```

        Args:
            year: 年份
            top_n: 排名
            industry_code: 行业代码
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.year = year
        self.top_n = top_n
        self.industry_code = industry_code
        self.size = size
        self.mapping = AnalystYearRankMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="分析师指数-年度排行榜")
        self.conditions = []

    def params_top_n(self) -> int:
        """"""
        if self.top_n is not None:
            return self.top_n
        else:
            if self.industry_code is not None:
                return 10
            else:
                return 100

    def params_filter(self) -> str:
        """"""
        if self.year:
            self.conditions.append(f'(YEAR="{self.year}")')
        else:
            raise ValueError("year is required.")
        if self.industry_code:
            self.conditions.append(f'(INDUSTRY_CODE="{self.industry_code}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "sortColumns": "YEAR_YIELD",
            "sortTypes": "-1",
            "pageSize": self.size,
            "reportName": "RPT_ANALYST_INDEX_RANK",
            "distinct": "ANALYST_CODE",
            "limit": f"top{self.params_top_n()}",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



from typing import Any, Dict, Optional, Literal, Callable
from wisecon.types import APIMainHolder, BaseMapping


__all__ = [
    "MainHolderMapping",
    "MainHolder",
]


class MainHolderMapping(BaseMapping):
    """字段映射 机构持股一览表"""
    columns: Dict = {
        "SECURITY_INNER_CODE": "证券内部代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "REPORT_DATE": "报告日期",
        "ORG_TYPE": "组织类型",
        "HOULD_NUM": "持有股数",
        "TOTAL_SHARES": "总股份数",
        "HOLD_VALUE": "持有市值",
        "FREESHARES_RATIO": "自由流通股比例",
        "HOLDCHA": "持仓变动",
        "HOLDCHA_NUM": "增仓股数",
        "HOLDCHA_RATIO": "增仓比例",
        "SECUCODE": "证券代码",
        "TOTALSHARES_RATIO": "总股份比例",
        "ORG_TYPE_NAME": "组织类型名称",
        "QCHANGE_RATE": "季度变动率",
        "FREE_MARKET_CAP": "自由流通市值",
        "FREE_SHARES": "自由流通股",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "HOLDCHA_VALUE": "增仓市值",
        "SECURITY_CODE": "证券代码",
        "FREESHARES_RATIO_CHANGE": "自由流通股比例变动",
        "TYPE_NUM": "类型编号"
    }


class MainHolder(APIMainHolder):
    """查询 机构持股一览表"""

    def __init__(
            self,
            holder: Optional[Literal["基金", "QFII", "社保", "券商", "保险", "信托"]] = None,
            status: Optional[Literal["全部", "增持", "减持"]] = "全部",
            date: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.main_holder import *

            data = MainHolder(holder="基金", date="2024-09-30").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder: 机构类型
            status: 持股变动情况
            date: 查询日期
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.holder = holder
        self.status = status
        self.date = date
        self.size = size
        self.mapping = MainHolderMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="机构持股一览表",)
        self.validate_date_is_end_if_quarter(date=date)

    def params_holder_type(self) -> int:
        """"""
        holder_mapping = {"基金": 1, "QFII": 2, "社保": 3, "券商": 4, "保险": 5, "信托": 6}
        if self.holder in holder_mapping:
            return holder_mapping[self.holder]
        else:
            raise ValueError(f"holder_code must be in {holder_mapping.keys()}")

    def params_status(self) -> int:
        """"""
        status_mapping = {"全部": 0, "增持": 1, "减持": 2}
        if self.status in status_mapping:
            return status_mapping[self.status]
        else:
            raise ValueError(f"status must be in {status_mapping.keys()}")

    def params(self) -> Dict:
        """"""
        params = {
            "date": self.date,
            "type": self.params_holder_type(),
            "zjc": self.params_status(),
            "sortField": "HOULD_NUM",
            "pageSize": self.size,
        }
        return self.base_param(update=params)



import json
from functools import partial
from typing import Any, List, Dict, Callable, Optional
from wisecon.types import BaseMapping, BaseRequestData
from wisecon.utils import jquery_mock_callback, time2int, filter_dict_by_key
from .base import other_headers


__all__ = [
    "FundHistMapping",
    "FundHist",
]


class FundHistMapping(BaseMapping):
    """字段映射 基金历史净值"""
    columns: Dict = {
        "FSRQ": "净值日期",
        "DWJZ": "单位净值",
        "LJJZ": "累计净值",
        "JZZZL": "日增长率",
        "SGZT": "申购状态",
        "SHZT": "赎回状态",
    }


class FundHist(BaseRequestData):
    """查询 基金历史净值"""
    def __init__(
            self,
            fund_code: str,
            start_date: Optional[str] = "",
            end_date: Optional[str] = "",
            limit: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.fund import FundHist

            data = FundHist(fund_code="000001", start_date="2020-01-01", end_date="2024-01-01", limit=10).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            fund_code: 基金代码
            start_date: 开始日期
            end_date: 结束日期
            limit: 返回条数
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.fund_code = fund_code
        self.start_date = start_date
        self.end_date = end_date
        self.limit = limit
        self.mapping = FundHistMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="text", description="基金历史净值", other_headers=other_headers)

    def base_url(self) -> str:
        """"""
        base_url = "https://api.fund.eastmoney.com/f10/lsjz"
        return base_url

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "callback": jquery_mock_callback(),
            "fundCode": self.fund_code,
            "pageIndex": "1",
            "pageSize": self.limit,
            "startDate": self.start_date,
            "endDate": self.end_date,
            "_": time2int(),
        }
        return params

    def clean_content(
            self,
            content: Optional[str] = None,
    ) -> List[Dict]:
        """"""
        content = content[content.find("(") + 1: content.rfind(")")]
        response = json.loads(content).pop("Data")
        data = response.pop("LSJZList")

        _filter_dict = partial(filter_dict_by_key, keys=list(self.mapping.columns.keys()))
        data = list(map(_filter_dict, data))
        self.metadata.response = response
        return data



import time
import requests
from bs4 import BeautifulSoup
from pydantic import Field
from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import ResponseData
from wisecon.utils import headers
from .base import *


__all__ = [
    "NewsQueryConfig",
    "News",
]


TypeNewsTheme = Literal[
    "财经导读", "产经新闻", "国内经济", "国际经济", "证券聚焦", "纵深调查",
    "经济时评", "产业透视", "商业观察", "股市评论", "商业资讯", "创业研究",
    "A股公司", "港股公司", "中概股公司", "海外公司",
]


class NewsQueryConfig(BaseRequestConfig):
    """"""
    size: Optional[int] = Field(default=20)
    theme: TypeNewsTheme = Field(default="财经导读")

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _theme(self):
        """"""
        theme_mapping = {
            "财经导读": "344",
            "产经新闻": "355",
            "国内经济": "350",
            "国际经济": "351",
            "证券聚焦": "353",
            "纵深调查": "363",
            "经济时评": "371",
            "产业透视": "372",
            "商业观察": "373",
            "股市评论": "374",
            "商业资讯": "670",
            "创业研究": "683",
            "A股公司": "349",
            "港股公司": "535",
            "中概股公司": "437",
            "海外公司": "440",
        }
        return theme_mapping.get(self.theme, "344")

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "client": "web",
            "biz": "web_news_col",
            "column": self._theme(),
            "order": 1,
            "needInteractData": 0,
            "page_index": 1,
            "page_size": self.size,
            "req_trace": str(int(time.time() * 1E3)),
            "fields": "code,showTime,title,mediaName,summary,image,url,uniqueUrl,Np_dst",
            "types": "1,20",
        }
        return params


class News(BaseRequestData):
    """新闻数据"""
    def __init__(
            self,
            query_config: Optional[NewsQueryConfig] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        :param size:
        :param theme: 新闻主题
            [
                "财经导读", "产经新闻", "国内经济", "国际经济", "证券聚焦", "纵深调查",
                "经济时评", "产业透视", "商业观察", "股市评论", "商业资讯", "创业研究",
                "A股公司", "港股公司", "中概股公司", "海外公司",
            ]
        :param query_config:
        :param verbose:
        :param logger:
        :param kwargs:
        """
        if query_config is None:
            self.query_config = NewsQueryConfig.model_validate(kwargs)
        else:
            self.query_config = query_config
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs

    def _base_url(self) -> str:
        """"""
        base_url = "https://np-listapi.eastmoney.com/comm/web/getNewsByColumns"
        return base_url

    def load_data(self) -> ResponseData:
        """
        :return:
        """
        metadata = self.request_json().get("data", {})
        data = metadata.pop("list")
        self.update_metadata(metadata)
        self._logger(msg=f"[{__class__.__name__}] Find {len(data)} reports.", color="green")
        return ResponseData(data=data, metadata=metadata)

    def update_metadata(self, metadata: Dict):
        """新闻"""
        metadata.update({
            "description": "新闻",
            "columns": {
                "code": "新闻编码",
                "showTime": "发布时间",
                "title": "标题",
                "mediaName": "媒体",
                "summary": "摘要",
                "image": "图片",
            }
        })

    def load_content(self, url: str) -> str:
        """"""
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            raise Exception(f"load content failed, status code: {response.status_code}")
        else:
            soup = BeautifulSoup(response.text)
            content = soup.find("div", class_="txtinfos")
            if len(content) == 0:
                raise Exception("load content failed, content is empty")
            else:
                paragraphs = content.find_all("p")
                return "\n\n".join([p.text for p in paragraphs])



import time
import requests
from datetime import datetime
from requests import Response
from pydantic import BaseModel, Field
from typing import List, Dict, Union, Optional, Literal
from wisecon.utils import headers, LoggerMixin
from .response_data import ResponseData, Metadata
from .mapping import BaseMapping


__all__ = [
    "assemble_url",
    "BaseRequestConfig",
    "BaseRequestData",
    "ValidateParams",
]


def assemble_url(base_url: str, params: Dict) -> str:
    """"""
    query_string = '&'.join([f"{key}={value}" for key, value in params.items()])
    request_url = f"{base_url}?{query_string}"
    return request_url


class BaseRequestConfig(BaseModel):
    """"""
    mapping: Optional[Dict[str, str]] = Field(default=None)

    def _current_time(self) -> str:
        """"""
        return str(int(time.time() * 1E3))

    def params(self) -> Dict:
        """"""
        return dict()


class ValidateParams:
    """"""
    security_code: Union[str, List[str]]

    def validate_date_format(self, date: Union[str, List[str]], _format: str = "%Y-%m-%d"):
        """"""
        if isinstance(date, str):
            date = [date]
        for d in date:
            if isinstance(d, str):
                try:
                    datetime.strptime(d, _format)
                except ValueError:
                    raise ValueError(f"Invalid date format: {d}. Expected format: {_format}")

    def validate_date_is_end_if_quarter(self, date: str):
        """"""
        _format = "%Y-%m-%d"
        self.validate_date_format(date, _format=_format)
        date = datetime.strptime(date, _format)
        mark = [
            (date.month == 3 and date.day == 31),
            (date.month == 6 and date.day == 30),
            (date.month == 9 and date.day == 30),
            (date.month == 12 and date.day == 31),
        ]
        if not any(mark):
            raise ValueError(f"Date must be the last day of the quarter, your date: {date}")

    def validate_code(self, code: Union[str, List[str]], length: int = 6):
        """"""
        if isinstance(code, str):
            code = [code]
        mark = all([len(c) == 6 for c in code])
        if not mark:
            raise ValueError(f"Security code must be {length} characters.")

    def validate_holder_code(self, code: str):
        """"""
        self.validate_code(code)

    def validate_security_code(self, code: Union[str, List[str]]):
        """"""
        self.validate_code(code)

    def validate_max_security_codes(self, security_code: Union[None, str, List[str]]) -> None:
        """"""
        if security_code:
            self.validate_code(code=security_code)
            if isinstance(security_code, list) and len(security_code) > 10:
                raise ValueError("security_code maximum length is 10.")

    def validate_size(self, size: int, max_size: int):
        """"""
        if size > max_size:
            raise ValueError(f"Size must be less than {max_size}, your size: {size}")

    def validate_params(self):
        """"""
        if hasattr(self, "security_code"):
            self.validate_security_code(self.security_code)
        if hasattr(self, "holder_code"):
            self.validate_holder_code(self.holder_code)


class BaseRequestData(LoggerMixin, ValidateParams):
    """"""
    query_config: Optional[BaseRequestConfig]
    headers: Optional[Dict]
    response_type: Literal["json", "text"]
    metadata: Optional[Metadata]
    mapping: Optional[BaseMapping]

    def request_set(
            self,
            _headers: Optional[Dict] = None,
            response_type: Optional[Literal["json", "text", "html"]] = "json",
            description: Optional[str] = "",
            other_headers: Optional[Dict] = None
    ):
        """"""
        self.headers = _headers if _headers else headers
        self.response_type = response_type
        self.metadata = Metadata(description=description, columns=self.mapping.columns)
        if other_headers:
            self.headers.update(other_headers)

    def base_url(self) -> str:
        """"""
        return ""

    def params(self) -> Dict:
        """"""
        return dict()

    def params_drop_none(self, params: Dict) -> Dict:
        """"""
        return {k: v for k, v in params.items() if v is not None}

    def request(self) -> Response:
        """"""
        base_url = self.base_url()
        params = self.params()
        self._logger(msg=f"[URL] {assemble_url(base_url, params)}\n", color="green")
        response = requests.get(base_url, params=params, headers=self.headers)
        return response

    def request_json(self) -> Dict:
        """"""
        response = self.request()
        return response.json()

    def request_text(self) -> str:
        """"""
        response = self.request()
        return response.text

    def request_html(self) -> bytes:
        """"""
        response = self.request()
        return response.content

    def data(self, data: List[Dict], metadata: Optional[Metadata]) -> ResponseData:
        """"""
        return ResponseData(data=data, metadata=metadata)

    def clean_html(
            self,
            html: Optional[bytes],
    ) -> List[Dict]:
        return []

    def clean_content(
            self,
            content: Optional[str],
    ) -> List[Dict]:
        return []

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        return []

    def load(self) -> ResponseData:
        """"""
        if self.response_type == "text":
            content = self.request_text()
            data = self.clean_content(content)
        elif self.response_type == "html":
            html = self.request_html()
            data = self.clean_html(html)
        elif self.response_type == "json":
            json_data = self.request_json()
            data = self.clean_json(json_data)
        else:
            raise ValueError(f"Invalid response type: {self.response_type}")
        return self.data(data=data, metadata=self.metadata)



import time
import requests
import pandas as pd
from typing import Dict
from wisecon.utils import LoggerMixin
from wisecon.types.headers import headers
from wisecon.types import ResponseData


__all__ = [
    "ConceptionMap"
]


class ConceptionMap(LoggerMixin):
    """"""
    base_url: str = "https://reportapi.eastmoney.com/report/bk"
    mapping_code: Dict[str, str] = {
        "地域板块": "020",
        "行业板块": "016",
        "概念板块": "007",
    }

    def __init__(self):
        """"""
        self.map_industry = self.list_industry()
        self.map_conception = self.list_conception()
        self.map_district = self.list_district()
        self.mapping_data = sum([
            self.map_conception.data, self.map_industry.data, self.map_district.data
        ], [])

    def _get_data(self, params: Dict) -> ResponseData:
        """"""
        response = requests.get(self.base_url, params=params, headers=headers.headers)
        metadata = response.json()
        data = metadata.pop("data")
        return ResponseData(data=data, metadata=metadata)

    def list_industry(self, ) -> ResponseData:
        """"""
        params = {"bkCode": "016", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)

    def list_conception(self) -> ResponseData:
        """"""
        params = {"bkCode": "007", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)

    def list_district(self) -> ResponseData:
        """"""
        params = {"bkCode": "020", "_": str(int(time.time() * 1E3))}
        return self._get_data(params)

    def get_code_by_name(self, name: str) -> pd.DataFrame:
        """"""
        df = pd.DataFrame(self.mapping_data)
        return df.loc[df.bkName.str.contains(name), ["bkCode", "bkName"]]

    def get_name_by_code(self, code: str) -> pd.DataFrame:
        """"""
        df = pd.DataFrame(self.mapping_data)
        return df.loc[df.bkCode == code, ["bkCode", "bkName"]]



import unittest
from wisecon.stock.market import *


class TestETFMarket(unittest.TestCase):
    """"""
    def test_etf_market(self):
        """"""
        data = CurrentMarket(market="ETF").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_lof_market(self):
        """"""
        data = CurrentMarket(market="LOF").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_close_market(self):
        """"""
        data = CurrentMarket(market="封闭基金").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_market_total_a(self):
        """"""
        data = CurrentMarket(market="上证A股").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_market_total_sh_a(self):
        """"""
        data = CurrentMarket(market="京市A股").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock_market_AB(self):
        """"""
        data = CurrentMarket(market="上证AB股比价").load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestLastDayMarketSummary(unittest.TestCase):
    """"""
    def test_last_day_market_summary(self):
        """"""
        data = LastDayMarketSummary().load()
        print(data.to_frame(chinese_column=True).to_markdown())




from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "ScheduledDisclosure",
    "ScheduledDisclosureMapping",
]


class ScheduledDisclosureMapping(BaseMapping):
    """字段映射 上市公司预约披露时间"""
    columns: Dict = {
        "SECURITY_CODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "REPORT_TYPE": "报告类型",
        "REPORT_YEAR": "报告年份",
        "FIRST_APPOINT_DATE": "首次任命日期",
        "FIRST_CHANGE_DATE": "首次变更日期",
        "SECOND_CHANGE_DATE": "第二次变更日期",
        "THIRD_CHANGE_DATE": "第三次变更日期",
        "ACTUAL_PUBLISH_DATE": "实际发布日期",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "SECURITY_TYPE": "证券类型",
        "TRADE_MARKET_CODE": "交易市场代码",
        "TRADE_MARKET": "交易市场",
        "REPORT_DATE": "报告日期",
        "APPOINT_PUBLISH_DATE": "任命发布日期",
        "RESIDUAL_DAYS": "剩余天数",
        "REPORT_TYPE_NAME": "报告类型名称",
        "IS_PUBLISH": "是否发布",
        "MARKET": "市场",
        "EITIME": "导入时间",
        "SECUCODE": "证券代码"
    }


class ScheduledDisclosure(APIDataV1RequestData):
    """查询 上市公司预约披露时间"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            industry_name: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import ScheduledDisclosure

            data = ScheduledDisclosure(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            industry_name: 行业名称
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.industry_name = industry_name
        self.mapping = ScheduledDisclosureMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司预约披露时间")
        self.conditions = []

    def params_filter(self) -> str:
        """
        :return:
        """
        self.filter_report_date()
        self.filter_security_code()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "FIRST_APPOINT_DATE,SECURITY_CODE",
            "sortTypes": "1,1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_PUBLIC_BS_APPOIN",
        }
        return self.base_param(params)



import re
import ast
from typing import Dict, List, Union, Tuple


__all__ = [
    "ParseBase",
]


class ParseBase:
    """"""
    matches: List[str] = None
    sparse_error: List[Tuple] = []

    @classmethod
    def greedy_mark(
            cls,
            string: str,
            mark_pattern: str,
    ) -> Union[List[List], List[Dict]]:
        """
        解析 value 贪婪模式，寻找第一个 'mark' 到最后一个 'mark' 之间的内容，适用于文本中只有 1 个 mark content 的解析。
        :param mark_pattern:
        :param string:
        :return:
        """
        cls.matches = re.findall(mark_pattern, string, flags=re.DOTALL)
        values = []
        for match in cls.matches:
            try:
                value = ast.literal_eval(match)
                values.append(value)
            except ValueError() as error:
                cls.sparse_error.append((match, f"Error: {error}"))
        return values

    @classmethod
    def eval_mark(
            cls,
            string: str,
            mark_pattern: str,
    ) -> Union[List[List], List[Dict]]:
        """
        解析 value 非贪婪模式，寻找任意 'mark' 之间 'mark' 之间的内容，适用于存在多个非嵌套模 value 的解析。
        :param mark_pattern:
        :param string:
        :return:

        example:
        """
        cls.matches = re.findall(mark_pattern, string, flags=re.DOTALL)
        values = []
        for match in cls.matches:
            try:
                value = ast.literal_eval(match)
                values.append(value)
            except ValueError() as error:
                cls.sparse_error.append((match, f"Error: {error}"))
        return values

    @classmethod
    def nested_data(
            cls,
            string: str,
            mark: tuple = ("{", "}"),
    ) -> Union[List[List], List[Dict]]:
        """
        解析嵌套的 values，基本适用于全部情况的 value 解析。
        :param mark:
        :param string:
        :return:

        example:
        """
        stack = []
        nested_values_idx = []

        for i, char in enumerate(string):
            if char == mark[0]:
                stack.append(i)
            elif char == mark[1]:
                if stack:
                    left_bracket_index = stack.pop()
                    if len(stack) == 0:
                        nested_values_idx.append((left_bracket_index, i + 1))

        nested_values = []
        for (start, end) in nested_values_idx:
            try:
                value = ast.literal_eval(string[start: end])
                nested_values.append(value)
            except ValueError() as error:
                cls.sparse_error.append((string[start: end], f"Error: {error}"))
        return nested_values



import unittest
from wisecon.report.news import *


class TestNews(unittest.TestCase):
    def test_news(self):
        news = News(size=20)
        data = news.load_data()

        print(data.metadata)
        self.assertIsNotNone(data)
        print(data.to_frame(columns=data.metadata.get("columns")).to_markdown())




from typing import Any, Dict, Callable, Optional
from wisecon.types import ResponseData, BaseMapping
from .base import MacroRequestData


__all__ = [
    "BoomIndexMapping",
    "BoomIndex",
]


class BoomIndexMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "BOOM_INDEX": "企业景气指数",
        "FAITH_INDEX": "企业家信心指数",
        "BOOM_INDEX_SAME": "企业景气指数（同比）",
        "BOOM_INDEX_SEQUENTIAL": "企业景气指数（环比）",
        "FAITH_INDEX_SAME": "企业家信心指数（同比）",
        "FAITH_INDEX_SEQUENTIAL": "企业家信心指数（环比）",
    }


class BoomIndex(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = BoomIndexMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="中国 企业景气及企业家信心指数")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "BOOM_INDEX", "FAITH_INDEX", "BOOM_INDEX_SAME",
            "BOOM_INDEX_SEQUENTIAL", "FAITH_INDEX_SAME", "FAITH_INDEX_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_BOOM_INDEX",
        }
        return self.base_param(params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "StockStatisticsMapping",
    "StockStatistics",
]


class StockStatisticsMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "TOTAL_SHARES_SH": "发行总股本(SH)",
        "TOTAL_MARKE_SH": "市价总值(SH)",
        "DEAL_AMOUNT_SH": "成交金额(SH)",
        "VOLUME_SH": "成交量(SH)",
        "HIGH_INDEX_SH": "A股最高综合股价指数(SH)",
        "LOW_INDEX_SH": "A股最低综合股价指数(SH)",
        "TOTAL_SZARES_SZ": "发行总股本(SZ)",
        "TOTAL_MARKE_SZ": "市价总值(SZ)",
        "DEAL_AMOUNT_SZ": "成交金额(SZ)",
        "VOLUME_SZ": "成交量(SZ)",
        "HIGH_INDEX_SZ": "A股最高综合股价指数(SZ)",
        "LOW_INDEX_SZ": "A股最低综合股价指数(SZ)",
    }
    
    
class StockStatistics(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = StockStatisticsMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 全国股票交易统计表")
    
    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "TOTAL_SHARES_SH", "TOTAL_MARKE_SH", "DEAL_AMOUNT_SH", "VOLUME_SH",
            "HIGH_INDEX_SH", "LOW_INDEX_SH", "TOTAL_SZARES_SZ", "TOTAL_MARKE_SZ", "DEAL_AMOUNT_SZ",
            "VOLUME_SZ", "HIGH_INDEX_SZ", "LOW_INDEX_SZ"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_STOCK_STATISTICS",
        }
        return self.base_param(update=params)



import unittest
from wisecon.fund.fund_value import FundValue


class TestFundValue(unittest.TestCase):
    """ 测试：基金当前净值 """
    def test_fund_list(self):
        data = FundValue(fund_code="000001").load()
        data.show_columns()
        print(data.to_frame().shape)
        print(data.to_frame(chinese_column=True).head().to_markdown())




from typing import Annotated, Optional, Dict


__all__ = [
    ""
]


def search_fund(
        fund_name: Annotated[Optional[str], '基金名称', True] = None,
        fund_code: Annotated[Optional[str], '基金代码', False] = None,
        max_: Annotated[Optional[int], '最大返回数量', False] = 10,
) -> str:
    """
    依据基金名称或者基金代码，获取基金代码、基金拼音简写、基金类型、基金拼音全称等信息

    :param fund_name: 基金名称
    :param fund_code: 基金代码
    :param max_: 最大返回数量
    :return:
    """
    columns = ['基金代码', '基金拼音简写', '基金名称', '基金类型', '基金拼音全称']
    df = pd.DataFrame(info, columns=columns, dtype=str)
    if fund_code:
        data = df[df["基金代码"] == fund_code].to_dict("records")
        return data
    elif fund_name:
        data = df[df["基金名称"].str.contains(fund_name)].to_dict("records")[:max_]
        return data
    else:
        raise ValueError("fund_code or fund_name must be provided")



def get_fund_history(
        fund_code: Annotated[str, '基金代码', True],
        start_date: Annotated[str, '查询开始日期', True],
        end_date: Annotated[str, '查询结束日期', True],
) -> str:
    """
    给定基金代码、查询开始与结束日期，查询日期内该基金的历史行情信息。
    :param fund_code: 基金代码
    :param end_date: 查询开始日期, yyyy-MM-dd
    :param start_date: 查询结束日期, yyyy-MM-dd
    :return:
    """
    columns = ['FSRQ', 'DWJZ', 'LJJZ', 'SDATE', 'ACTUALSYI', 'NAVTYPE', 'JZZZL', 'SGZT',
               'SHZT', 'FHFCZ', 'FHFCBZ', 'DTYPE', 'FHSP']
    col_mapping = {
        "FSRQ": "净值日期", "DWJZ": "单位净值", "LJJZ": "累计净值",
        "JZZZL": "日增长率", "SGZT": "申购状态", "SHZT": "赎回状态",
    }
    jq_callback = jquery_mock_callback()
    url = f"https://api.fund.eastmoney.com/f10/lsjz?callback={jq_callback}" \
          f"&fundCode={fund_code}&pageIndex={1}&pageSize={100}" \
          f"&startDate={start_date}&endDate={end_date}&_={get_current_timestamp()}"

    text = get_url(url)
    null, false = None, False
    fund_info = eval(text.replace(jq_callback, ''))
    df_fund = pd.DataFrame(fund_info.get('Data').get('LSJZList'))
    df_fund = df_fund[columns]
    df_fund = df_fund[list(col_mapping.keys())].rename(columns=col_mapping)
    data = df_fund.to_markdown(index=False)
    return data


def get_fund_company(
        company_name: Annotated[Optional[str], '基金公司名称', True] = None,
        company_code: Annotated[Optional[str], '基金公司代码', True] = None,
) -> List[Dict]:
    """
    给定基金公司代码查询基金公司名称，或给定基金公司名称查询基金公司代码

    :param company_name: 基金公司名称
    :param company_code: 基金公司代码
    :return:
    """
    url = "http://fund.eastmoney.com/js/jjjz_gs.js"
    info = get_url(url)
    data = ParseList.greedy_list(info)[0]
    columns = ['基金公司代码', '基金公司名称']
    df_data = pd.DataFrame(data, columns=columns)
    if company_code:
        data = df_data[df_data["基金公司代码"] == company_code].to_dict("records")
        return data
    elif company_name:
        data = df_data[df_data["基金公司名称"].str.contains(company_name)].to_dict("records")
        return data
    else:
        raise ValueError("fund_code or fund_name must be provided")


def get_current_fund(
        fund_code: Annotated[str, '基金代码', True] = '000001',
) -> Union[Dict, str]:
    """
    依据基金代码，获取当前基金的净值、涨幅等信息。
    :param fund_code: 基金代码
    :return: 基金当前数据
    """
    url = f"http://fundgz.1234567.com.cn/js/{fund_code}.js"
    text = get_url(url)
    data = ParseDict.eval_dict(text)
    if len(data) > 0:
        return CurrentFundData(**data[0]).map_dict()
    else:
        return f"未查询到 {fund_code} 的数据。"

def get_fund_basic_info(
        fund_code: Annotated[str, '基金代码', True] = '000001',
) -> Dict:
    """
    依据基金代码获取基金的基本信息。基本信息包括:
        ['基金全称', '基金简称', '基金代码', '基金类型', '发行日期', '成立日期/规模', '资产规模', '份额规模', '基金管理人',
        '基金托管人', '基金经理人', '成立来分红', '管理费率', '托管费率',  '销售服务费率', '最高认购费率', '最高申购费率',
        '最高赎回费率', '业绩比较基准', '跟踪标的']

    :param fund_code: 基金代码
    :return: 基金基本信息
    """
    url = f"https://fundf10.eastmoney.com/jbgk_{fund_code}.html"
    text = get_url(url)
    soup = BeautifulSoup(text, 'lxml')
    table_info = soup.find_all('table', {'class': 'info w790'})[0]
    th = [item.text for item in table_info.find_all('th')]
    td = [item.text for item in table_info.find_all('td')]
    fund_basic_info = dict(zip(th, td))
    return fund_basic_info



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "TotalRetailMapping",
    "TotalRetail",
]


class TotalRetailMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "RETAIL_TOTAL": "当月(亿元)",
        "RETAIL_TOTAL_SAME": "同比增长",
        "RETAIL_TOTAL_SEQUENTIAL": "环比增长",
        "RETAIL_TOTAL_ACCUMULATE": "累计(亿元)",
        "RETAIL_ACCUMULATE_SAME": "累计同比增长",
    }


class TotalRetail(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = TotalRetailMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 社会消费品零售总额 （每年2月公布当年1-2月累计值）")

    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "TIME", "RETAIL_TOTAL", "RETAIL_TOTAL_SAME", "RETAIL_TOTAL_SEQUENTIAL",
            "RETAIL_TOTAL_ACCUMULATE", "RETAIL_ACCUMULATE_SAME"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_TOTAL_RETAIL",
        }
        return params



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "FreeHolderCoop",
    "FreeHolderCoopMapping",
]


class FreeHolderCoopMapping(BaseMapping):
    """字段映射 上市公司十大流通股东股东协同"""
    columns: Dict = {
        "HOLDER_NEW": "新的股东",
        "HOLDER_NAME": "股东名称",
        "HOLDER_TYPE": "股东类型",
        "COOPERAT_HOLDER_NEW": "合作股东代码",
        "COOPERAT_HOLDER_NAME": "合作股东姓名",
        "COOPERAT_HOLDER_TYPE": "合作股东类型",
        "COOPERAT_NUM": "合作持股数",
        "COOPERAT_SECURITYDATE": "合作证券日期",
        "PINGJIE": "评介"
    }


class FreeHolderCoop(APIDataV1RequestData):
    """查询 上市公司十大流通股东股东协同"""
    def __init__(
            self,
            holder_name: Optional[str] = None,
            holder_type: Optional[Literal["个人", "基金", "QFII", "社保", "券商", "信托"]] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.holder import *

            data = FreeHolderCoop(size=20).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            holder_name: 股东名称
            holder_type: 股东类型 `["个人", "基金", "QFII", "社保", "券商", "信托"]`
            size: 返回条数
            verbose: 是否显示日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.holder_name = holder_name
        self.holder_type = holder_type
        self.size = size
        self.mapping = FreeHolderCoopMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司十大流通股东股东协同")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        if self.holder_type:
            self.conditions.append(f'(HOLDER_TYPE="{self.holder_type}")')
        if self.holder_name:
            self.conditions.append(f'(HOLDER_NAME+like+"%{self.holder_name}%")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "COOPERAT_NUM,HOLDER_NEW,COOPERAT_HOLDER_NEW",
            "sortTypes": "-1,-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_COOPFREEHOLDER",
        }
        return self.base_param(params)



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import APICListMapping, APICListRequestData


__all__ = [
    "PlateCodeMapping",
    "PlateCode",
]


class PlateCodeMapping(APICListMapping):
    """字段映射 行业、概念、地区编码数据"""


class PlateCode(APICListRequestData):
    """查询 行业、概念、地区编码数据"""
    def __init__(
            self,
            plate_type: Optional[Literal["行业", "概念", "地区"]] = None,
            size: Optional[int] = 500,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.capital_flow import *

            # 1. 查询有哪些概念板块
            data = PlateCode(plate_type="概念").load()
            data.to_frame(chinese_column=True)

            # 2. 查询有哪些地区板块
            data = PlateCode(plate_type="地区").load()
            data.to_frame(chinese_column=True)

            # 3. 查询有哪些行业板块
            data = PlateCode(plate_type="行业").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            plate_type: 板块类型, `["行业", "概念", "地区"]`
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志函数
            **kwargs: 其他参数
        """
        self.plate_type = plate_type
        self.size = size
        self.mapping = PlateCodeMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="行业、概念、地区编码数据", )

    def params_plate_type(self) -> str:
        """"""
        plate_mapping = {
            "行业": "m:90+t:2",
            "概念": "m:90+t:3",
            "地区": "m:90+t:1",
        }
        if self.plate_type in plate_mapping:
            return plate_mapping[self.plate_type]
        else:
            return plate_mapping["行业"]

    def params(self) -> Dict:
        """"""
        params = {
            "fid": "f62",
            "po": 1,
            "pz": self.size,
            "pn": 1,
            "np": 1,
            "fs": self.params_plate_type(),
            "fields": "f12,f13,f14,f62",
        }
        return params



import duckdb
import warnings
import pandas as pd
from typing import Callable, Literal, Optional
from wisecon.utils.logger import LoggerMixin
from duckdb_engine import DuckDBEngineWarning
warnings.filterwarnings("ignore", category=DuckDBEngineWarning)


__all__ = ["DuckDB"]


class DuckDB(LoggerMixin):
    def __init__(
            self,
            database: str,
            logger: Optional[Callable] = None,
            verbose: Optional[bool] = None,
            **kwargs,
    ):
        self.conn = duckdb.connect(database=database, **kwargs)
        self.logger = logger
        self.verbose = verbose

    def execute(self, query: str):
        return self.conn.execute(query).fetchall()

    def close(self):
        self.conn.close()

    def save_dataframe(
            self,
            df: pd.DataFrame,
            name: str,
            if_exists: Literal["fail", "replace", "append"] = "replace",
    ) -> None:
        """"""
        df.to_sql(name=name, con=self.conn, if_exists=if_exists, index=False)




import unittest
from pprint import pprint
from wisecon.stock.index import *


class TestHolderStockAnalysis(unittest.TestCase):

    def test_columns(self):
        """"""
        data = IndexStock(size=5, verbose=True).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)

    def test_load(self):
        """
        :return:
        """
        data = IndexStock(size=5, verbose=True).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """
        :return:
        """
        for index_name in ["沪深300", "上证50", "中证500", "科创50"]:
            print(index_name)
            data = IndexStock(index_name=index_name, size=5).load()
            print(data.to_frame(chinese_column=True).to_markdown())
            print("=" * 100)



import os
import unittest
import inspect
from wisecon.stock.etf import *
from wisecon.agent import *
from wisecon.types.agent import Function

# def gold_etf(
#         market:
# ):
#     """"""
#     data = ETFGoldHistory(market="ETF白银").load()
#     tool_list
#


class TestToolCall(unittest.TestCase):
    """"""
    def test_tools_sync(self):
        """"""
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[ETFGoldHistory])
        agent = BaseAgent(model="glm-4-flash", base_url=base_url, tools=tools, api_key_name="ZHIPU_API_KEY")
        completion = agent.chat(prompt="查询2023年12月1日的ETF白银历史数据")

    def test_tools_sync_chat(self):
        """"""
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[ETFGoldHistory])
        agent = BaseAgent(model="glm-4-flash", base_url=base_url, tools=tools, api_key_name="ZHIPU_API_KEY")
        completion = agent.chat(prompt="你好")

    def test_tools_sse(self):
        """"""
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[ETFGoldHistory])
        agent = BaseAgent(
            model="glm-4-flash", base_url=base_url, tools=tools, api_key_name="ZHIPU_API_KEY")
        completion = agent.chat(prompt="查询2023年12月1日的ETF白银历史数据", stream=True)
        answer = ""
        for chunk in completion:
            if chunk.choices[0].delta.content is None:
                answer += str(chunk.choices[0].delta.tool_calls[0].function.model_dump())
            else:
                answer += chunk.choices[0].delta.content
        print(answer)

    def test_tools_sse_chat(self):
        """"""
        base_url = "https://open.bigmodel.cn/api/paas/v4"
        tools = Tools(tools=[ETFGoldHistory])
        agent = BaseAgent(
            model="glm-4-flash", base_url=base_url, tools=tools, api_key_name="ZHIPU_API_KEY")
        completion = agent.chat(prompt="你好", stream=True)
        for chunk in completion:
            print(chunk)

    def test_etf_gold_bai(self):
        """"""
        data = ETFGoldHistory(market="ETF白银").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_etf_gold(self):
        """"""
        data = ETFGoldHistory(market="ETF黄金", date="2024-10-25").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_etf_gold_date(self):
        """"""
        data = ETFGoldHistory(market="ETF黄金", start_date="2024-10-25").load()
        print(data.to_frame(chinese_column=True).to_markdown())



from typing import Dict, Optional, List
from wisecon.types import APIDataV1RequestData
from wisecon.utils import time2int


__all__ = [
    "MacroRequestData"
]


class MacroRequestData(APIDataV1RequestData):
    """"""
    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "pageNumber": "1",
            "source": "WEB",
            "client": "WEB",
            "_": time2int(),
        }
        params.update(update)
        return params



from typing import Any, Dict, Literal, Callable, Optional, Annotated
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "StockBalance",
    "StockBalanceMapping",
]


TypeMarket = Literal["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]


class StockBalanceMapping(BaseMapping):
    """字段映射 上市公司资产负债报表"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "INDUSTRY_CODE": "行业代码",
        "ORG_CODE": "机构代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "INDUSTRY_NAME": "行业名称",
        "MARKET": "市场",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "TRADE_MARKET_CODE": "交易市场代码",
        "DATE_TYPE_CODE": "日期类型代码",
        "REPORT_TYPE_CODE": "报告类型代码",
        "DATA_STATE": "数据状态",
        "NOTICE_DATE": "公告日期",
        "REPORT_DATE": "报告日期",
        "TOTAL_ASSETS": "总资产",
        "FIXED_ASSET": "固定资产",
        "MONETARYFUNDS": "货币资金",
        "MONETARYFUNDS_RATIO": "货币资金占比",
        "ACCOUNTS_RECE": "应收账款",
        "ACCOUNTS_RECE_RATIO": "应收账款占比",
        "INVENTORY": "存货",
        "INVENTORY_RATIO": "存货占比",
        "TOTAL_LIABILITIES": "总负债",
        "ACCOUNTS_PAYABLE": "应付账款",
        "ACCOUNTS_PAYABLE_RATIO": "应付账款占比",
        "ADVANCE_RECEIVABLES": "预收账款",
        "ADVANCE_RECEIVABLES_RATIO": "预收账款占比",
        "TOTAL_EQUITY": "总权益",
        "TOTAL_EQUITY_RATIO": "总权益占比",
        "TOTAL_ASSETS_RATIO": "总资产占比",
        "TOTAL_LIAB_RATIO": "总负债占比",
        "CURRENT_RATIO": "流动比率",
        "DEBT_ASSET_RATIO": "资产负债率",
        "CASH_DEPOSIT_PBC": "现金存款（央行）",
        "CDP_RATIO": "现金存款占比",
        "LOAN_ADVANCE": "贷款预支",
        "LOAN_ADVANCE_RATIO": "贷款预支占比",
        "AVAILABLE_SALE_FINASSET": "可供出售金融资产",
        "ASF_RATIO": "可供出售金融资产占比",
        "LOAN_PBC": "央行贷款",
        "LOAN_PBC_RATIO": "央行贷款占比",
        "ACCEPT_DEPOSIT": "接受存款",
        "ACCEPT_DEPOSIT_RATIO": "接受存款占比",
        "SELL_REPO_FINASSET": "出售回购金融资产",
        "SRF_RATIO": "出售回购金融资产占比",
        "SETTLE_EXCESS_RESERVE": "结算超额准备金",
        "SER_RATIO": "超额准备金占比",
        "BORROW_FUND": "借款资金",
        "BORROW_FUND_RATIO": "借款资金占比",
        "AGENT_TRADE_SECURITY": "代理交易证券",
        "ATS_RATIO": "代理交易证券占比",
        "PREMIUM_RECE": "应收保费",
        "PREMIUM_RECE_RATIO": "应收保费占比",
        "SHORT_LOAN": "短期借款",
        "SHORT_LOAN_RATIO": "短期借款占比",
        "ADVANCE_PREMIUM": "预收保费",
        "ADVANCE_PREMIUM_RATIO": "预收保费占比"
    }


class StockBalance(APIDataV1RequestData):
    """查询 上市公司资产负债报表"""
    def __init__(
            self,
            security_code: Annotated[Optional[str], "证券代码", False] = None,
            market: Annotated[Optional[TypeMarket], '市场名称: ["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]', False] = None,
            industry_name: Annotated[Optional[str], "行业名称", False] = None,
            size: Annotated[Optional[int], "返回数据量", False] = 50,
            start_date: Annotated[Optional[str], "开始日期", False] = None,
            end_date: Annotated[Optional[str], "结束日期", False] = None,
            date: Annotated[Optional[str], "日期: 一般是季度末日期", False] = None,
            verbose: Annotated[Optional[bool], "", False] = False,
            logger: Annotated[Optional[Callable], "", False] = None,
            **kwargs: Annotated[Any, "", False]
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import StockBalance

            # 查询第三季度上市公司资产负债表数据
            data = StockBalance(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            market: 市场: `["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]`
            industry_name: 行业名称
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.market = market
        self.industry_name = industry_name
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = StockBalanceMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司资产负债报表")

    def params_tread_market(self) -> str:
        """"""
        market_mapping = {
            "沪深A股": 'TRADE_MARKET_CODE!="069001017")',
            "沪市A股": 'TRADE_MARKET_CODE+in+("069001001001","069001001003","069001001006"))',
            "科创板": '(TRADE_MARKET_CODE="069001001006")',
            "深市A股": 'TRADE_MARKET_CODE+in+("069001002001","069001002002","069001002003","069001002005"))',
            "创业板": '(TRADE_MARKET_CODE="069001002002")',
            "京市A股": 'TRADE_MARKET_CODE="069001017")',
        }
        return market_mapping[self.market]

    def params_filter(self) -> str:
        """"""
        conditions = ['(SECURITY_TYPE_CODE in ("058001001","058001008"))']
        if self.start_date:
            conditions.append(f"(REPORT_DATE>='{self.start_date}')")
        if self.end_date:
            conditions.append(f"(REPORT_DATE<='{self.end_date}')")
        if self.date:
            conditions.append(f"(REPORT_DATE='{self.date}')")
        if self.market:
            conditions.append(self.params_tread_market())
        if self.industry_name:
            conditions.append(f'(INDUSTRY_NAME="{self.industry_name}")')
        if self.security_code:
            conditions.append(f'(SECURITY_CODE="{self.security_code}")')
        return "".join(conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_DMSK_FN_BALANCE",
        }
        return self.base_param(params)




import time
import random
import requests
from pydantic import BaseModel
from typing import List, Dict, Union, Literal, Optional
from wisecon.types.request_data import BaseRequestData, assemble_url
from wisecon.types.response_data import ResponseData


__all__ = [
    "TypeReport",
    "ReportData",
    "APIReportRequest",
]


TypeReport = Literal["个股研报", "行业研报", "策略报告", "宏观研究", "券商晨报", "*"]


class ReportData(BaseModel):
    """"""
    code: str
    content: Optional[bytes] = None
    error: Optional[str] = None


class APIReportRequest(BaseRequestData):
    """"""
    page_no: int
    reports_data: List[ReportData]
    q_type: Optional[Union[int, str]] = None
    report_type: Optional[TypeReport] = "*"

    def reports_type(self):
        """"""
        report_types = ["个股研报", "行业研报", "策略报告", "宏观研究", "券商晨报"]
        if self.report_type is None or self.report_type == "*":
            self.q_type = "*"
        elif self.report_type in report_types:
            self.q_type = report_types.index(self.report_type)

    def base_url(self) -> str:
        """jg/dg/list"""
        url = "https://reportapi.eastmoney.com/report/"
        if self.q_type is None:
            self.reports_type()
        if self.q_type in [0, 1,]:
            url += "list"
        else:
            url += "dg"
        return url

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {}
        params.update(update)
        return params

    def random_cb(self) -> str:
        """"""
        return str(int(random.random() * 1E7 + 1))

    def current_time(self) -> str:
        """"""
        return str(int(time.time() * 1E3))

    def data_page(self):
        """"""
        base_url = self.base_url()
        params = self.params()
        self._logger(msg=f"[URL] {assemble_url(base_url, params)}\n", color="green")
        response = requests.get(base_url, params=params, headers=self.headers)
        json_data = response.json()
        total_page = json_data.get("TotalPage")
        page_no = json_data.get("pageNo")
        return json_data, total_page, page_no

    def request(self) -> List[Dict]:
        """"""
        json_data, total_page, page_no = self.data_page()
        data = json_data.pop("data")
        self.metadata.response = json_data
        while page_no < total_page:
            self.page_no += 1
            json_data, total_page, page_no = self.data_page()
            page_data = json_data.pop("data")
            data.extend(page_data)
            self.metadata.response = json_data
        return data

    def load(self) -> ResponseData:
        """"""
        return self.data(data=self.request(), metadata=self.metadata)



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "MarginTradingSummaryMapping",
    "MarginTradingSummary",
]


class MarginTradingSummaryMapping(BaseMapping):
    """字段映射 融资融券交易总量-市场合计"""
    columns: Dict = {
        "RZYE": "融资余额",
        "RQYL": "融券余额",
        "RZMRE": "融资买入额",
        "DIM_DATE": "日期",
        "XOB_MARKET_0001": "市场",
        "RQYE": "融券余量",
        "SCDM": "市场代码",
        "RQMCL": "融券卖出量",
        "NEW": "新增",
        "ZDF": "涨跌幅",
        "LTSZ": "流通市值",
        "RZYEZB": "融资余额占比",
        "RZMRE3D": "3日融资买入额",
        "RZMRE5D": "5日融资买入额",
        "RZMRE10D": "10日融资买入额",
        "RZCHE": "融资偿还额",
        "RZCHE3D": "3日融资偿还额",
        "RZCHE5D": "5日融资偿还额",
        "RZCHE10D": "10日融资偿还额",
        "RZJME": "融资交易额",
        "RZJME3D": "3日融资交易额",
        "RZJME5D": "5日融资交易额",
        "RZJME10D": "10日融资交易额",
        "RQMCL3D": "3日融券卖出量",
        "RQMCL5D": "5日融券卖出量",
        "RQMCL10D": "10日融券卖出量",
        "RQCHL": "融券持有量",
        "RQCHL3D": "3日融券持有量",
        "RQCHL5D": "5日融券持有量",
        "RQCHL10D": "10日融券持有量",
        "RQJMG": "融券交易量",
        "RQJMG3D": "3日融券交易量",
        "RQJMG5D": "5日融券交易量",
        "RQJMG10D": "10日融券交易量",
        "ZDF3D": "3日涨跌幅",
        "ZDF5D": "5日涨跌幅",
        "ZDF10D": "10日涨跌幅",
        "RZRQYE": "融资融券余额",
        "RZRQYECZ": "融资融券余额调整",
    }


class MarginTradingSummary(APIDataV1RequestData):
    """查询 融资融券交易总量-市场合计"""
    def __init__(
            self,
            market: Literal["全部", "沪市", "深市", "京市"] = "全部",
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            size: Optional[int] = 100,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.margin import *

            # 查询 融资融券交易总量-市场合计
            data = MarginTradingSummary(market="全部").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场类型
            start_date: 开始日期
            end_date: 结束日期
            date: 日期
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.market = market
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.size = size
        self.mapping = MarginTradingSummaryMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="融资融券交易总量-市场合计")
        self.conditions = []

    def params_report_name(self) -> str:
        """"""
        if self.market == "全部":
            return "RPTA_RZRQ_LSHJ"
        else:
            return "RPTA_WEB_RZRQ_LSSH"

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="DIM_DATE")
        market_mapping = {
            "沪市": "007", "深市": "001", "京市": "002"
        }
        if self.market in market_mapping:
            self.conditions.append(f'(scdm="{market_mapping[self.market]}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "reportName": self.params_report_name(),
            "sortColumns": "dim_date",
            "sortTypes": "-1",
            "pageSize": self.size,
            "filter": self.params_filter(),
            "pageNo": "1",
        }
        return self.base_param(update=params)



import re
from typing import List, Dict, Tuple
from .parse import ParseBase


__all__ = ["ParseList"]


class ParseList(ParseBase):
    """"""
    matches: List[str] = None
    pattern: str = r'\[.*?\]'
    greedy_pattern: str = r'\[.*\]'
    mark: Tuple[str] = ("[", "]")

    @classmethod
    def greedy_list(cls, string: str) -> List[List]:
        """"""
        return cls.greedy_mark(string=string, mark_pattern=cls.greedy_pattern)

    @classmethod
    def eval_list(cls, string: str) -> List[List]:
        """"""
        return cls.eval_mark(string=string, mark_pattern=cls.pattern)

    @classmethod
    def nested_list(cls, string: str) -> List[List]:
        """"""
        return cls.nested_data(string=string, mark=cls.mark)




from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "StockIncome",
    "StockIncomeMapping",
]


TypeMarket = Literal["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]


class StockIncomeMapping(BaseMapping):
    """字段映射 上市公司利润报表"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "INDUSTRY_CODE": "行业代码",
        "ORG_CODE": "机构代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "INDUSTRY_NAME": "行业名称",
        "MARKET": "市场",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "TRADE_MARKET_CODE": "交易市场代码",
        "DATE_TYPE_CODE": "日期类型代码",
        "REPORT_TYPE_CODE": "报告类型代码",
        "DATA_STATE": "数据状态",
        "NOTICE_DATE": "公告日期",
        "REPORT_DATE": "报告日期",
        "PARENT_NETPROFIT": "净利润",
        "TOTAL_OPERATE_INCOME": "营业总收入",
        "TOTAL_OPERATE_COST": "营业总支出",
        "TOE_RATIO": "营业利润率",
        "OPERATE_COST": "营业支出",
        "OPERATE_EXPENSE": "营业费用",
        "OPERATE_EXPENSE_RATIO": "营业费用占比",
        "SALE_EXPENSE": "销售费用",
        "MANAGE_EXPENSE": "管理费用",
        "FINANCE_EXPENSE": "财务费用",
        "OPERATE_PROFIT": "营业利润",
        "TOTAL_PROFIT": "总利润",
        "INCOME_TAX": "所得税",
        "OPERATE_INCOME": "营业收入",
        "INTEREST_NI": "利息净收入",
        "INTEREST_NI_RATIO": "利息净收入占比",
        "FEE_COMMISSION_NI": "手续费及佣金净收入",
        "FCN_RATIO": "手续费及佣金净收入占比",
        "OPERATE_TAX_ADD": "营业税金及附加",
        "MANAGE_EXPENSE_BANK": "银行管理费用",
        "FCN_CALCULATE": "手续费及佣金净收入计算",
        "INTEREST_NI_CALCULATE": "利息净收入计算",
        "EARNED_PREMIUM": "已赚保费",
        "EARNED_PREMIUM_RATIO": "已赚保费占比",
        "INVEST_INCOME": "投资收益",
        "SURRENDER_VALUE": "退保价值",
        "COMPENSATE_EXPENSE": "赔偿费用",
        "TOI_RATIO": "营业总收入同比",
        "OPERATE_PROFIT_RATIO": "营业利润率",
        "PARENT_NETPROFIT_RATIO": "净利润率",
        "DEDUCT_PARENT_NETPROFIT": "扣除后归属于母公司的净利润",
        "DPN_RATIO": "扣除后净利润率"
    }


class StockIncome(APIDataV1RequestData):
    """查询 上市公司利润报表"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            market: Optional[TypeMarket] = None,
            industry_name: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import StockIncome

            data = StockIncome(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            market: 市场: `["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]`
            industry_name: 行业名称
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.market = market
        self.industry_name = industry_name
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = StockIncomeMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司利润报表")
        self.conditions = []

    def params_tread_market(self) -> str:
        """"""
        market_mapping = {
            "沪深A股": 'TRADE_MARKET_CODE!="069001017")',
            "沪市A股": 'TRADE_MARKET_CODE+in+("069001001001","069001001003","069001001006"))',
            "科创板": '(TRADE_MARKET_CODE="069001001006")',
            "深市A股": 'TRADE_MARKET_CODE+in+("069001002001","069001002002","069001002003","069001002005"))',
            "创业板": '(TRADE_MARKET_CODE="069001002002")',
            "京市A股": 'TRADE_MARKET_CODE="069001017")',
        }
        return market_mapping[self.market]

    def params_filter(self) -> str:
        """"""
        self.conditions.append('(SECURITY_TYPE_CODE in ("058001001","058001008"))')
        self.filter_report_date()
        self.filter_security_code()
        if self.market:
            self.conditions.append(self.params_tread_market())
        if self.industry_name:
            self.conditions.append(f'(INDUSTRY_NAME="{self.industry_name}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_DMSK_FN_INCOME",
        }
        return self.base_param(params)



import unittest
from wisecon.stock.plate_mapping import *


class TestPlateMapping(unittest.TestCase):
    def test_plate_industry(self):
        data = PlateCode(plate_type="行业").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_zone(self):
        data = PlateCode(plate_type="地区").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_plate_conception(self):
        data = PlateCode(plate_type="概念").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_industry_code(self):
        data = IndustryCode().load()
        print(data.to_markdown(chinese_column=True))



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "CurrencySupplyMapping",
    "CurrencySupply",
]


class CurrencySupplyMapping(BaseMapping):
    """字段映射 货币供应量"""
    columns: Dict = {
        "REPORT_DATE": "报告日期",
        "TIME": "时间",
        "BASIC_CURRENCY": "货币和准货币(M2)",
        "BASIC_CURRENCY_SAME": "M2-同比",
        "BASIC_CURRENCY_SEQUENTIAL": "M2-环比",
        "CURRENCY": "货币(M1)",
        "CURRENCY_SAME": "M1-同比",
        "CURRENCY_SEQUENTIAL": "M1-环比",
        "FREE_CASH": "流通中的现金(M0)",
        "FREE_CASH_SAME": "M0-同比",
        "FREE_CASH_SEQUENTIAL": "M0-环比",
    }


class CurrencySupply(MacroRequestData):
    """查询 中国货币供应量"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.macro import *
            data = CurrencySupply(size=20).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            size: 最大数据量
            verbose: 是否打印日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.size = size
        self.mapping = CurrencySupplyMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(
            response_type="json",
            description="中国货币供应量",
        )

    def params(self) -> Dict:
        """"""
        columns = [
            "REPORT_DATE", "TIME", "BASIC_CURRENCY", "BASIC_CURRENCY_SAME", "BASIC_CURRENCY_SEQUENTIAL",
            "CURRENCY", "CURRENCY_SAME", "CURRENCY_SEQUENTIAL", "FREE_CASH", "FREE_CASH_SAME",
            "FREE_CASH_SEQUENTIAL"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_ECONOMY_CURRENCY_SUPPLY",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "MarketValuationMapping",
    "MarketValuation",
]


class MarketValuationMapping(BaseMapping):
    """字段映射 市场整体估值"""
    columns: Dict = {
        "TRADE_MARKET_CODE": "市场代码",
        "TRADE_DATE": "交易日期",
        "CLOSE_PRICE": "指数值",
        "CHANGE_RATE": "涨跌幅(%)",
        "SECURITY_INNER_CODE": "内部代码",
        "LISTING_ORG_NUM": "个股总数",
        "TOTAL_SHARES": "总股本(股)",
        "FREE_SHARES": "流通股本(股)",
        "TOTAL_MARKET_CAP": "总市值(元)",
        "FREE_MARKET_CAP": "流通市值(元)",
        "PE_TTM_AVG": "平均市盈率(PE-TTM)"
    }


class MarketValuation(APIDataV1RequestData):
    """查询 市场整体估值
    ```json
    {
        "000300": "沪深300",
        "000001": "上证指数",
        "000688": "科创50",
        "399001": "深证指数",
        "399006": "创业板指数",
    }
    ```
    """
    def __init__(
            self,
            market: Optional[Literal["沪深两市", "沪市主板", "科创板", "深市主板", "创业板",]] = "沪深两市",
            start_date: Optional[str] = "2020-10-08",
            end_date: Optional[str] = None,
            size: Optional[int] = 50,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            data = MarketValuation(market=market, size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            market: 市场名称
            start_date: 开始日期
            end_date: 结束日期
            size: 返回条数
            verbose: 是否显示日志
            logger: 自定义日志
            **kwargs: 其他参数
        """
        self.market = market
        self.start_date = start_date
        self.end_date = end_date
        self.size = size
        self.mapping = MarketValuationMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="市场整体估值")
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date], )

    def params_market(self):
        """"""
        market_mapping = {
            "沪深两市": "000300",
            "沪市主板": "000001",
            "科创板": "000688",
            "深市主板": "399001",
            "创业板": "399006",
        }
        self.conditions.append(f"(TRADE_MARKET_CODE=\"{market_mapping[self.market]}\")")

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="TRADE_DATE")
        self.params_market()
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": -1,
            "pageSize": self.size,
            "reportName": "RPT_VALUEMARKET",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional, Literal
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "ImpInterestMapping",
    "ImpInterest",
]


description = """
CHIBOR即中国银行间同业拆借利率（China Interbank Offered Rate），
1996年1月3日在中国人民银行总行的领导下，依托中国外汇交易中心的网络，
12家商业银行总行和15家融资中心开始进行人民币联网拆借交易，
全国性的同业拆借市场成立，并产生了CHIBOR。
CHIBOR值以各银行同业拆借实际交易利率的加权平均值来确定。"""


class ImpInterestMapping(BaseMapping):
    """"""
    columns: Dict = {
        "REPORT_DATE": "时间",
        "REPORT_PERIOD": "周期",
        "IR_RATE": "利率(%)",
        "CHANGE_RATE": "涨跌(BP)",
        "INDICATOR_ID": "周期代码",
        "MARKET": "市场",
        "MARKET_CODE": "市场代码",
        "CURRENCY": "币种",
        "CURRENCY_CODE": "币种代码",
    }


class ImpInterest(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            market: Literal["上海", "中国", "伦敦", "欧洲", "香港", "新加坡"] = "上海",
            currency_code: Literal["CNY", "HKD", "USD", "GBP", "JPY", "EUR", "SGD"] = "CNY",
            indicator: Literal["1d", "1w", "2w", "1m", "3m", "6m", "9m", "1y"] = "1d",
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        :param size:
        :param currency_code: 币种
            ["CNY", "HKD", "USD", "GBP", "JPY", "EUR", "SGD"]
        :param market: 市场
            {"上海": "001", "中国": "002", "伦敦": "003", "欧洲": "004", "香港": "005", "新加坡": "006"}
        :param indicator: 指标周期
            {
                "1d": "001",
                "1w": "101", "2w": "102",
                "1m": "201", "2m": "202", "3m": "203", "4m": "204", "5m": "205",
                "6m": "206", "7m": "207", "8m": "208", "9m": "209", "10m": "210", "11m": "211",
                "1y": "301",
            }
        :param query_config:
        :param verbose:
        :param logger:
        :param kwargs:
        """
        self.size = size
        self.currency_code = currency_code
        self.market = market
        self.indicator = indicator
        self.mapping = ImpInterestMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description=description)
    
    def _query_filter(self) -> str:
        """"""
        market_mapping = {"上海": "001", "中国": "002", "伦敦": "003", "欧洲": "004", "香港": "005", "新加坡": "006"}
        indicator_mapping = {
            "1d": "001",
            "1w": "101", "2w": "102",
            "1m": "201", "2m": "202", "3m": "203", "4m": "204", "5m": "205",
            "6m": "206", "7m": "207", "8m": "208", "9m": "209", "10m": "210", "11m": "211",
            "1y": "301",
        }
        _filter = f'(MARKET_CODE="{market_mapping.get(self.market)}")(CURRENCY_CODE="CNY")(INDICATOR_ID="{indicator_mapping.get(self.indicator)}")'
        return _filter
    
    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "REPORT_DATE", "REPORT_PERIOD", "IR_RATE", "CHANGE_RATE", "INDICATOR_ID", "LATEST_RECORD",
            "MARKET", "MARKET_CODE", "CURRENCY", "CURRENCY_CODE",
        ]
        params = {
            "filter": self._query_filter(),
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "REPORT_DATE",
            "sortTypes": "-1",
            "reportName": "RPT_IMP_INTRESTRATEN",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping, APIAnalystInvest


__all__ = [
    "AnalystIndexMapping",
    "AnalystIndex",
]


class AnalystIndexMapping(BaseMapping):
    """字段映射 分析师指数"""
    columns: Dict = {
        "ANALYST_CODE": "分析师代码",
        "ANALYST_NAME": "分析师姓名",
        "ORG_CODE": "机构代码",
        "ORG_NAME": "机构名称",
        "INDEX_PVALUE": "指数基准值",
        "INDEX_VALUE": "指数值",
        "TRADE_DATE": "交易日期",
        "UPDATE_DATE": "更新时间",
        "INDEX_HVALUE": "指数历史值",
        "INDEX_CHANGE": "指数变动",
        "INDEX_CHANGE_RATE": "指数变动率"
    }


class AnalystIndex(APIAnalystInvest):
    """查询 分析师指数"""
    def __init__(
            self,
            analyst_code: Optional[str] = None,
            size: Optional[int] = 100,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.analyst import *

            data = AnalystIndex(analyst_code="11000280036").load()
            print(data.to_markdown(chinese_column=True))
            ```

        Args:
            analyst_code: 分析师代码
            size: 返回数据量
            verbose: 是否打印日志
            logger: 自定义日志打印函数
            **kwargs: 其他参数
        """
        self.analyst_code = analyst_code
        self.size = size
        self.mapping = AnalystIndexMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="分析师指数")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        self.conditions.append(f'(ANALYST_CODE="{self.analyst_code}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """"""
        params = {
            "sortColumns": "TRADE_DATE",
            "sortTypes": "-1",
            "pageSize": self.size,
            "reportName": "RPT_RESEARCHER_DETAILS",
            "filter": self.params_filter(),
        }
        return self.base_param(update=params)



from typing import Any, Dict, Callable, Optional
from wisecon.types import BaseMapping
from .base import MacroRequestData


__all__ = [
    "OilPriceMapping",
    "OilPrice",
]


class OilPriceMapping(BaseMapping):
    """"""
    columns: Dict = {
        "DIM_DATE": "调整日期",
        "VALUE": "汽油价格(元/吨)",
        "CY_JG": "柴油价格(元/吨)",
        "QY_FD": "汽油涨跌",
        "CY_FD": "柴油涨跌",
    }


class OilPrice(MacroRequestData):
    """"""
    def __init__(
            self,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        self.size = size
        self.mapping = OilPriceMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="中国 汽柴油历史调价信息")
        
    def params(self) -> Dict:
        """
        :return:
        """
        columns = [
            "DIM_DATE", "VALUE", "CY_JG", "QY_FD", "CY_FD"
        ]
        params = {
            "columns": ",".join(columns),
            "pageSize": self.size,
            "sortColumns": "DIM_DATE",
            "sortTypes": "-1",
            "reportName": "RPTA_WEB_YJ_BD",
        }
        return self.base_param(update=params)



from typing import List, Dict, Optional
from wisecon.utils import time2int
from wisecon.types.request_data import BaseRequestData


__all__ = [
    "APICListRequestData",
    "APIStockFFlowKLineRequestData",
    "APIStockFFlowDayLineRequestData",
    "APIUListNPRequestData",
    "APIDataRequestData",
    "APIDataV1RequestData",
    "APIStockKline",
    "APIStockKlineWithSSE",
    "APIMainHolder",
    "APIAnalystInvest",
    "APIMainHolderDetail",
    "APIStockTrends2",
    "APIMarketSummary",
]


class APICListRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://push2.eastmoney.com/api/qt/clist/get"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "pn": 1,
            "po": 1,
            "np": 1,
            "fltt": 2,
            "invt": 2,
            "dect": 1,
            "wbp2u": "|0|0|0|web",
            "_": time2int(),
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.get("data", {})
        data = response.pop("diff")
        self.metadata.response = response
        return data


class APIStockFFlowKLineRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://push2.eastmoney.com/api/qt/stock/fflow/kline/get"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "_": time2int(),
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        columns = list(self.mapping.columns.keys())
        response = json_data.get("data", {})
        data = response.pop("klines")
        data = [dict(zip(columns, item.split(","))) for item in data]
        self.metadata.response = response
        return data


class APIStockFFlowDayLineRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        columns = list(self.mapping.columns.keys())
        response = json_data.get("data", {})
        data = response.pop("klines")
        data = [dict(zip(columns, item.split(","))) for item in data]
        self.metadata.response = response
        return data


class APIUListNPRequestData(BaseRequestData):
    """"""
    def base_url(self) -> str:
        return "https://push2.eastmoney.com/api/qt/ulist.np/get"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.get("data", {})
        data = response.pop("diff")
        self.metadata.response = response
        return data


class APIDataRequestData(BaseRequestData):
    """"""
    conditions: Optional[List[str]]
    security_code: Optional[str]
    start_date: Optional[str]
    end_date: Optional[str]
    date: Optional[str]

    def base_url(self) -> str:
        """"""
        return "https://datacenter-web.eastmoney.com/api/data/get"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.get("result", {})
        try:
            data = response.pop("data")
            self.metadata.response = response
            return data
        except Exception as e:
            raise ValueError(f"Error in cleaning json data; response: {json_data}")

    def filter_report_date(self, date_name: Optional[str] = "REPORT_DATE"):
        """"""
        if hasattr(self, "start_date") and self.start_date:
            self.conditions.append(f"({date_name}>='{self.start_date}')")
        if hasattr(self, "end_date") and self.end_date:
            self.conditions.append(f"({date_name}<='{self.end_date}')")
        if hasattr(self, "date") and self.date:
            self.conditions.append(f"({date_name}='{self.date}')")

    def filter_security_code(self):
        """"""
        if self.security_code:
            self.conditions.append(f'(SECURITY_CODE="{self.security_code}")')

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "extraCols": "",
            "filter": "",
            "sr": "",
            "st": "",
            "token": "",
            "var": "",
            "source": "QuoteWeb",
            "client": "WEB",
            "_": time2int(),
        }
        params.update(update)
        return params


class APIDataV1RequestData(BaseRequestData):
    """"""
    conditions: Optional[List[str]]
    security_code: Optional[str]
    start_date: Optional[str]
    end_date: Optional[str]
    date: Optional[str]

    def base_url(self) -> str:
        """"""
        return "https://datacenter-web.eastmoney.com/api/data/v1/get"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.get("result", {})
        try:
            data = response.pop("data")
            self.metadata.response = response
            return data
        except Exception as e:
            raise ValueError(f"Error in cleaning json data; response: {json_data}")

    def filter_report_date(self, date_name: Optional[str] = "REPORT_DATE"):
        """"""
        if hasattr(self, "start_date") and self.start_date:
            self.conditions.append(f"({date_name}>='{self.start_date}')")
        if hasattr(self, "end_date") and self.end_date:
            self.conditions.append(f"({date_name}<='{self.end_date}')")
        if hasattr(self, "date") and self.date:
            self.conditions.append(f"({date_name}='{self.date}')")

    def filter_security_code(self):
        """"""
        if self.security_code:
            self.conditions.append(f'(SECURITY_CODE="{self.security_code}")')

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "columns": "ALL",
            "pageNumber": "1",
            "quoteColumns": "",
            "source": "WEB",
            "client": "WEB",
            "_": time2int(),
        }
        params.update(update)
        return params


class APIStockKline(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://push2his.eastmoney.com/api/qt/stock/kline/get"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        response = json_data.get("data", {})
        data = response.pop("klines")
        self.metadata.response = response

        def trans_kline_data(line: str) -> Dict:
            """"""
            line_data = line.split(",")
            return dict(zip(list(self.mapping.columns.keys()), line_data))

        data = list(map(trans_kline_data, data))
        return data


class APIStockKlineWithSSE(BaseRequestData):
    """
    URL - 1
        https://push2his.eastmoney.com/api/qt/stock/kline/get
        https://push2his.eastmoney.com/api/qt/stock/kline/sse

    URL - 2
        https://push2.eastmoney.com/api/qt/stock/get
        https://push2.eastmoney.com/api/qt/stock/sse
    """
    def base_url(self) -> str:
        """"""
        return "https://push2.eastmoney.com/api/qt/stock/get"

    def base_sse(self) -> str:
        """"""
        return "https://push2.eastmoney.com/api/qt/stock/sse"

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""

        def clean_data(data: Dict) -> Dict:
            """"""
            columns = [
                "f11", "f13", "f15", "f17", "f19",
                "f31", "f33", "f35", "f37", "f39",
                "f191",
            ]
            for key, value in data.items():
                if key in columns:
                    data.update({key: value / 100})
            return data

        data = list(map(clean_data, [json_data.pop("data")]))
        self.metadata.response = json_data
        return data


class APIMainHolder(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://data.eastmoney.com/dataapi/zlsj/list"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "sortDirec": "1",
            "pageNum": "1",
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        data = json_data.pop("data", {})
        self.metadata.response = json_data
        return data


class APIAnalystInvest(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://data.eastmoney.com/dataapi/invest/list"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "columns": "ALL",
            "source": "WEB",
            "client": "WEB",
            "pageNumber": "1",
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        response = json_data.pop("result", {})
        data = response.pop("data", [])
        self.metadata.response = response
        return data


class APIMainHolderDetail(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://data.eastmoney.com/dataapi/zlsj/detail"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "SHCode": "",
            "sortDirec": "1",
            "pageNum": "1",
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        data = json_data.pop("data", {})
        self.metadata.response = json_data
        return data


class APIStockTrends2(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://push2his.eastmoney.com/api/qt/stock/trends2/get"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {
            "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13,f17",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
            "mpi": "1000",
            "iscr": "0",
            "iscca": "0",
            "wbp2u": "|0|0|0|web",
        }
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """
        Args:
            json_data:

        Returns:


            response = json_data.get("data", {})
            data = response.pop("klines")
            data = [dict(zip(columns, item.split(","))) for item in data]
            self.metadata.response = response
        """
        columns = list(self.mapping.columns.keys())
        response = json_data.pop("data", {})
        data = response.pop("trends", [])
        data = [dict(zip(columns, item.split(","))) for item in data]
        self.metadata.response = response
        return data


class APIMarketSummary(BaseRequestData):
    """"""
    def base_url(self) -> str:
        """"""
        return "https://quote.eastmoney.com/newapi/sczm"

    def base_param(self, update: Dict) -> Dict:
        """"""
        params = {}
        params.update(update)
        return params

    def clean_json(
            self,
            json_data: Optional[Dict],
    ) -> List[Dict]:
        """"""
        data_name = ["ss", "cyb", "hs"]
        data = [json_data.pop(d_name) for d_name in data_name]
        self.metadata.response = json_data
        return data


# todo: 封装请求数据类



from typing import Any, Dict, Optional, Callable
from wisecon.types import APIDataV1RequestData, BaseMapping


__all__ = [
    "InstitutionTradeRankMapping",
    "InstitutionTradeRank",
]


class InstitutionTradeRankMapping(BaseMapping):
    """字段映射 机构买卖每日统计"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "SECURITY_CODE": "证券代码",
        "TRADE_DATE": "交易日期",
        "CLOSE_PRICE": "收盘价格",
        "CHANGE_RATE": "变动率",
        "BUY_TIMES": "买入次数",
        "SELL_TIMES": "卖出次数",
        "BUY_AMT": "买入金额",
        "SELL_AMT": "卖出金额",
        "NET_BUY_AMT": "净买入金额",
        "ACCUM_AMOUNT": "累计金额",
        "RATIO": "比例",
        "TURNOVERRATE": "换手率",
        "FREECAP": "自由流通市值",
        "EXPLANATION": "说明",
        "D1_CLOSE_ADJCHRATE": "D1日收盘调整幅度",
        "D2_CLOSE_ADJCHRATE": "D2日收盘调整幅度",
        "D3_CLOSE_ADJCHRATE": "D3日收盘调整幅度",
        "D5_CLOSE_ADJCHRATE": "D5日收盘调整幅度",
        "D10_CLOSE_ADJCHRATE": "D10日收盘调整幅度",
        "MARKET": "市场",
        "TRADE_MARKET_CODE": "交易市场代码",
        "BUY_COUNT": "买入计数",
        "SELL_COUNT": "卖出计数",
        "SECURITY_TYPE_CODE": "证券类型代码"
    }


class InstitutionTradeRank(APIDataV1RequestData):
    """查询 机构买卖每日统计"""

    def __init__(
            self,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            size: Optional[int] = 20,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.ranking import *

            data = InstitutionTradeRank(start_date="2024-10-23").load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            start_date: 开始日期
            end_date: 结束日期
            size: 数据条数
            verbose: 是否打印日志
            logger: 日志记录器
            **kwargs: 其他参数
        """
        self.start_date = start_date
        self.end_date = end_date
        self.size = size
        self.mapping = InstitutionTradeRankMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(description="机构买卖每日统计",)
        self.conditions = []
        self.validate_date_format(date=[start_date, end_date], _format="%Y-%m-%d")

    def params_filter(self) -> str:
        """"""
        self.filter_report_date(date_name="TRADE_DATE")
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "pageSize": self.size,
            "sortColumns": "NET_BUY_AMT,TRADE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1,1",
            "reportName": "RPT_ORGANIZATION_TRADE_DETAILSNEW",
        }
        return self.base_param(update=params)



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "StockCashFlow",
    "StockCashFlowMapping",
]


TypeMarket = Literal["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]


class StockCashFlowMapping(BaseMapping):
    """字段映射 上市公司现金流量报表"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "INDUSTRY_CODE": "行业代码",
        "ORG_CODE": "机构代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "INDUSTRY_NAME": "行业名称",
        "MARKET": "市场",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "TRADE_MARKET_CODE": "交易市场代码",
        "DATE_TYPE_CODE": "日期类型代码",
        "REPORT_TYPE_CODE": "报告类型代码",
        "DATA_STATE": "数据状态",
        "NOTICE_DATE": "公告日期",
        "REPORT_DATE": "报告日期",
        "NETCASH_OPERATE": "经营活动净现金流量",
        "NETCASH_OPERATE_RATIO": "经营活动净现金流量占比",
        "SALES_SERVICES": "销售服务收入",
        "SALES_SERVICES_RATIO": "销售服务收入占比",
        "PAY_STAFF_CASH": "支付员工现金",
        "PSC_RATIO": "支付员工现金占比",
        "NETCASH_INVEST": "投资活动净现金流量",
        "NETCASH_INVEST_RATIO": "投资活动净现金流量占比",
        "RECEIVE_INVEST_INCOME": "收到投资收益",
        "RII_RATIO": "收到投资收益占比",
        "CONSTRUCT_LONG_ASSET": "长期资产建设",
        "CLA_RATIO": "长期资产建设占比",
        "NETCASH_FINANCE": "融资活动净现金流量",
        "NETCASH_FINANCE_RATIO": "融资活动净现金流量占比",
        "CCE_ADD": "现金及现金等价物增加额",
        "CCE_ADD_RATIO": "现金及现金等价物增加额占比",
        "CUSTOMER_DEPOSIT_ADD": "客户存款增加额",
        "CDA_RATIO": "客户存款增加额占比",
        "DEPOSIT_IOFI_OTHER": "其他存款",
        "DIO_RATIO": "其他存款占比",
        "LOAN_ADVANCE_ADD": "贷款预支增加额",
        "LAA_RATIO": "贷款预支增加额占比",
        "RECEIVE_INTEREST_COMMISSION": "收到利息及佣金",
        "RIC_RATIO": "收到利息及佣金占比",
        "INVEST_PAY_CASH": "投资支付现金",
        "IPC_RATIO": "投资支付现金占比",
        "BEGIN_CCE": "期初现金及现金等价物",
        "BEGIN_CCE_RATIO": "期初现金及现金等价物占比",
        "END_CCE": "期末现金及现金等价物",
        "END_CCE_RATIO": "期末现金及现金等价物占比",
        "RECEIVE_ORIGIC_PREMIUM": "收到原保险保费",
        "ROP_RATIO": "收到原保险保费占比",
        "PAY_ORIGIC_COMPENSATE": "支付原保险赔偿",
        "POC_RATIO": "支付原保险赔偿占比"
    }


class StockCashFlow(APIDataV1RequestData):
    """查询 上市公司现金流量报表"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            market: Optional[TypeMarket] = None,
            industry_name: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import StockCashFlow

            data = StockCashFlow(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            market: 市场: `["沪深A股", "沪市A股", "科创板", "深市A股", "创业板", "京市A股"]`
            industry_name: 行业名称
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.market = market
        self.industry_name = industry_name
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.mapping = StockCashFlowMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司现金流量报表")
        self.conditions = []

    def params_tread_market(self) -> str:
        """"""
        market_mapping = {
            "沪深A股": 'TRADE_MARKET_CODE!="069001017")',
            "沪市A股": 'TRADE_MARKET_CODE+in+("069001001001","069001001003","069001001006"))',
            "科创板": '(TRADE_MARKET_CODE="069001001006")',
            "深市A股": 'TRADE_MARKET_CODE+in+("069001002001","069001002002","069001002003","069001002005"))',
            "创业板": '(TRADE_MARKET_CODE="069001002002")',
            "京市A股": 'TRADE_MARKET_CODE="069001017")',
        }
        return market_mapping[self.market]

    def params_filter(self) -> str:
        """"""
        self.conditions.append('(SECURITY_TYPE_CODE in ("058001001","058001008"))')
        self.filter_report_date()
        self.filter_security_code()
        if self.market:
            self.conditions.append(self.params_tread_market())
        if self.industry_name:
            self.conditions.append(f'(INDUSTRY_NAME="{self.industry_name}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_DMSK_FN_CASHFLOW",
        }
        return self.base_param(params)



from typing import Dict, List, Optional


__all__ = [
    'filter_dict_by_key',
    'filter_str_by_mark',
]


def filter_dict_by_key(d: Dict, keys: List[str]) -> Dict:
    return {k: v for k, v in d.items() if k in keys}


def filter_str_by_mark(
        s: str,
        start: Optional[str] = "{",
        end: Optional[str] = "}",
        keep_mark: Optional[bool] = True
) -> str:
    start_id = s.find(start)
    end_id = s.rfind(end)
    if keep_mark:
        return s[start_id: end_id + len(end)]
    else:
        return s[start_id + len(start): end_id]



from pydantic import BaseModel, Field
from typing import Literal, Dict


__all__ = [
    "Message",
    "ToolMessage"
]


class Message(BaseModel):
    role: str
    content: str

    def to_dict(self) -> Dict:
        return self.model_dump()


class ToolMessage(Message):
    """"""
    role: Literal["tool"] = Field(default="tool", description="角色")
    content: str = Field(default=None, description="对话内容")
    tool_call_id: str = Field(default=None, description="工具调用ID")



import unittest
from wisecon.types.columns import StockFeatures


class TestColumns(unittest.TestCase):

    def test_stock_features(self):
        stock_features = StockFeatures()
        stock_features._mapping(columns=["f31"])

    def test_stock_features_default(self):
        stock_features = StockFeatures()
        stock_features._mapping()

    def test_stock_features_all(self):
        stock_features = StockFeatures()
        stock_features._mapping(columns=[])



from typing import Any, Dict, Literal, Callable, Optional
from wisecon.types import BaseMapping, APIDataV1RequestData


__all__ = [
    "EarnForcast",
    "EarnForcastMapping",
]


class EarnForcastMapping(BaseMapping):
    """字段映射 上市公司业绩预告"""
    columns: Dict = {
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "ORG_CODE": "机构代码",
        "NOTICE_DATE": "公告日期",
        "REPORT_DATE": "报告日期",
        "PREDICT_FINANCE_CODE": "预测财务指标代码",
        "PREDICT_FINANCE": "预测财务指标",
        "PREDICT_AMT_LOWER": "预测金额下限",
        "PREDICT_AMT_UPPER": "预测金额上限",
        "ADD_AMP_LOWER": "增长幅度下限",
        "ADD_AMP_UPPER": "增长幅度上限",
        "PREDICT_CONTENT": "预测内容",
        "CHANGE_REASON_EXPLAIN": "变动原因说明",
        "PREDICT_TYPE": "预测类型",
        "PREYEAR_SAME_PERIOD": "去年同期金额",
        "TRADE_MARKET": "交易市场",
        "TRADE_MARKET_CODE": "交易市场代码",
        "SECURITY_TYPE": "证券类型",
        "SECURITY_TYPE_CODE": "证券类型代码",
        "INCREASE_JZ": "增加金额",
        "FORECAST_JZ": "预测金额",
        "FORECAST_STATE": "预测状态",
        "IS_LATEST": "是否最新",
        "PREDICT_RATIO_LOWER": "预测比率下限",
        "PREDICT_RATIO_UPPER": "预测比率上限",
        "PREDICT_HBMEAN": "预测均值"
    }
    predict_finance_mapping: Dict = {
        "归母净利润": "004",
        "扣非净利润": "005",
        "每股收益": "003",
        "营业总收入": "006"
    }


class EarnForcast(APIDataV1RequestData):
    """查询 上市公司业绩预告"""
    def __init__(
            self,
            security_code: Optional[str] = None,
            size: Optional[int] = 50,
            start_date: Optional[str] = None,
            end_date: Optional[str] = None,
            date: Optional[str] = None,
            forcast_state: Optional[Literal["reduction", "increase"]] = None,
            predict_finance: Optional[Literal["归母净利润", "扣非净利润", "每股收益", "营业总收入"]] = None,
            verbose: Optional[bool] = False,
            logger: Optional[Callable] = None,
            **kwargs: Any
    ):
        """
        Notes:
            ```python
            from wisecon.stock.financial import EarnForcast

            data = EarnForcast(date="2024-09-30", size=5).load()
            data.to_frame(chinese_column=True)
            ```

        Args:
            security_code: 证券代码
            size: 数据条数据
            start_date: 开始日期
            end_date: 结束日期
            date: 指定日期
            forcast_state: 预测状态 `["reduction", "increase"]`
            predict_finance: 预测指标 `["归母净利润", "扣非净利润", "每股收益", "营业总收入"]`
            verbose: 是否打印日志
            logger: 日志对象
            **kwargs: 其他参数
        """
        self.security_code = security_code
        self.size = size
        self.start_date = start_date
        self.end_date = end_date
        self.date = date
        self.forcast_state = forcast_state
        self.predict_finance = predict_finance
        self.mapping = EarnForcastMapping()
        self.verbose = verbose
        self.logger = logger
        self.kwargs = kwargs
        self.request_set(response_type="json", description="上市公司业绩预告")
        self.conditions = []

    def params_filter(self) -> str:
        """"""
        self.filter_report_date()
        self.filter_security_code()
        if self.forcast_state:
            self.conditions.append(f'(FORECAST_STATE="{self.forcast_state}")')
        if self.predict_finance:
            self.conditions.append(f'(PREDICT_FINANCE_CODE="{self.mapping.predict_finance_mapping[self.predict_finance]}")')
        return "".join(self.conditions)

    def params(self) -> Dict:
        """
        :return:
        """
        params = {
            "filter": self.params_filter(),
            "sortColumns": "NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": self.size,
            "pageNumber": 1,
            "reportName": "RPT_PUBLIC_OP_NEWPREDICT",
        }
        return self.base_param(params)



from typing import Optional, Union, List, Dict
from pydantic import BaseModel, Field


__all__ = [
    "StockFeatures"
]


class StockFeatures(BaseModel):
    """
    Stock features
    """
    # f1: Optional[Union[int, float]] = Field(default=None, description="open")
    # f2: Optional[Union[int, float]] = Field(default=None, description="close")
    # f3: Optional[Union[int, float]] = Field(default=None, description="high")
    # f4: Optional[Union[int, float]] = Field(default=None, description="low")
    # f5: Optional[Union[int, float]] = Field(default=None, description="涨跌幅")
    # f6: Optional[Union[int, float]] = Field(default=None, description="涨跌额")
    # f7: Optional[Union[int, float]] = Field(default=None, description="买五(价格)")
    # f8: Optional[Union[int, float]] = Field(default=None, description="买五(价格)")
    # f9: Optional[Union[int, float]] = Field(default=None, description="买五(价格)")
    # f10: Optional[Union[int, float]] = Field(default=None, description="买五(价格)")

    f11: Optional[Union[int, float]] = Field(default=None, description="买五(价格)")
    f12: Optional[Union[int, float]] = Field(default=None, description="买五(数量)")
    f13: Optional[Union[int, float]] = Field(default=None, description="买四(价格)")
    f14: Optional[Union[int, float]] = Field(default=None, description="买四(数量)")
    f15: Optional[Union[int, float]] = Field(default=None, description="买三(价格)")
    f16: Optional[Union[int, float]] = Field(default=None, description="买三(数量)")
    f17: Optional[Union[int, float]] = Field(default=None, description="买二(价格)")
    f18: Optional[Union[int, float]] = Field(default=None, description="买二(数量)")
    f19: Optional[Union[int, float]] = Field(default=None, description="买一(价格)")
    f20: Optional[Union[int, float]] = Field(default=None, description="买一(数量)")
    f31: Optional[Union[int, float]] = Field(default=None, description="卖五(价格)")
    f32: Optional[Union[int, float]] = Field(default=None, description="卖五(数量)")
    f33: Optional[Union[int, float]] = Field(default=None, description="卖四(价格)")
    f34: Optional[Union[int, float]] = Field(default=None, description="卖四(数量)")
    f35: Optional[Union[int, float]] = Field(default=None, description="卖三(价格)")
    f36: Optional[Union[int, float]] = Field(default=None, description="卖三(数量)")
    f37: Optional[Union[int, float]] = Field(default=None, description="卖二(价格)")
    f38: Optional[Union[int, float]] = Field(default=None, description="卖二(数量)")
    f39: Optional[Union[int, float]] = Field(default=None, description="卖一(价格)")
    f40: Optional[Union[int, float]] = Field(default=None, description="卖一(数量)")
    f43: Optional[Union[int, float]] = Field(default=None, description="最新")
    f44: Optional[Union[int, float]] = Field(default=None, description="最高")
    f45: Optional[Union[int, float]] = Field(default=None, description="最低")
    f46: Optional[Union[int, float]] = Field(default=None, description="今开")
    f47: Optional[Union[int, float]] = Field(default=None, description="总手")
    f48: Optional[float] = Field(default=None, description="金额")
    f49: Optional[float] = Field(default=None, description="外盘")
    f50: Optional[Union[int, float]] = Field(default=None, description="量比")
    f51: Optional[Union[int, float]] = Field(default=None, description="涨停")
    f52: Optional[Union[int, float]] = Field(default=None, description="跌停")
    f57: Optional[str] = Field(default=None, description="股票代码")
    f58: Optional[str] = Field(default=None, description="股票简称")
    # f59: 2,
    f60: Optional[str] = Field(default=None, description="昨收")
    f71: Optional[str] = Field(default=None, description="均价")
    f84: Optional[Union[int, float]] = Field(default=None, description="总股本")
    f85: Optional[Union[int, float]] = Field(default=None, description="流通股")
    # f86: 1727681655,
    f92: Optional[float] = Field(default=None, description="每股净资产")
    # f107: 0,
    # f108: 1.288050952,
    # f111: 80,
    # f116: 24551601900.0,
    # f117: 6137910000.0,
    # f152: 2,
    # f161: Optional[Union[int, float]] Field(default=None, description="涨跌") 62498,
    f162: Optional[Union[int, float]] = Field(default=None, description="PE(动态:总市值除以全年预估净利润,例如当前一季度净利润1000万,则预估全年净利润4000万)")
    f163: Optional[Union[int, float]] = Field(default=None, description="PE(静态:总市值除以上一年度净利润)")
    f164: Optional[Union[int, float]] = Field(default=None, description="PE(滚动:最新价除以最近4个季度的每股收益)")
    f167: Optional[Union[int, float]] = Field(default=None, description="市净")
    f168: Optional[Union[int, float]] = Field(default=None, description="换手")
    f169: Optional[Union[int, float]] = Field(default=None, description="涨跌")
    f170: Optional[Union[int, float]] = Field(default=None, description="涨幅")
    f171: Optional[Union[int, float]]= Field(default=None, description="振幅")
    # f177: 72,
    f191: Optional[Union[int, float]] = Field(default=None, description="委比")
    f192: Optional[Union[int, float]] = Field(default=None, description="委差")
    # f256: -,
    # f257: 0,
    # f260: 12,
    # f261: 457200.0,
    # f262: -,
    # f269: -,
    # f270: 0,
    # f277: 64439900.0,
    # f278: 16110000.0,
    # f279: 1,
    # f285: -,
    # f286: 0,
    # f288: 0,
    # f292: 13,
    # f294: 1,
    # f295: 0,
    # f301: 38100,
    f531: Optional[Union[int, float]] = Field(default=None, description="f531")
    # f734: N长联科技,
    # f747: -,
    # f748: 0,

    # def mapping(self):
    #     """"""
    #     properties = self.model_json_schema().get("properties")
    #     mapping = dict()

    def _mapping(self, columns: List[str]) -> Dict:
        """"""
        properties = self.model_json_schema().get("properties")
        mapping = dict()
        for column in columns:
            column_info = properties.get(column)
            if column_info is not None:
                description = column_info.get("description")
                mapping.update({column: description})
        return mapping

    def tick_columns(self) -> Dict:
        """"""
        columns = [
            "f11", "f12", "f13", "f14", "f15", "f16", "f17", "f18", "f19", "f20",
            "f31", "f32", "f33", "f34", "f35", "f36", "f37", "f38", "f39", "f40",
            "f191", "f192", "f531",
        ]
        columns_mapping = self._mapping(columns)
        return columns_mapping

    def company_core_columns(self):
        """"""



import unittest
from wisecon.stock.margin import *


class TestMarginTradeSummary(unittest.TestCase):
    """"""

    def test_margin_trade_summary(self):
        """"""
        data = MarginTradingSummary().load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_summary_sh(self):
        """"""
        data = MarginTradingSummary(market="沪市").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_summary_sz(self):
        """"""
        data = MarginTradingSummary(market="深市").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_summary_js(self):
        """"""
        data = MarginTradingSummary(market="京市").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_summary_date(self):
        """"""
        data = MarginTradingSummary(market="京市", start_date="2024-10-10").load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestMarginTradeAccount(unittest.TestCase):
    """"""
    def test_margin_trade_account_day(self):
        """"""
        data = MarginTradingAccount(cycle="day").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_account_month(self):
        """"""
        data = MarginTradingAccount(cycle="month").load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestMarginTradePlate(unittest.TestCase):
    """"""
    def test_margin_trade_plate_industry(self):
        """"""
        data = MarginTradingPlate(plate_name="行业", cycle=1).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_plate_conception(self):
        """"""
        data = MarginTradingPlate(plate_name="概念", cycle=1).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_plate_zone(self):
        """"""
        data = MarginTradingPlate(plate_name="地域", cycle=1).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_plate_zone_3day(self):
        """"""
        data = MarginTradingPlate(plate_name="地域", cycle=3, verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_plate_zone_5day(self):
        """"""
        data = MarginTradingPlate(plate_name="地域", cycle=5, verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_plate_zone_date(self):
        """"""
        data = MarginTradingPlate(plate_name="地域", cycle=5, date="2024-10-10").load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestMarginTradeStock(unittest.TestCase):
    """
    融资融券交易明细:
        个股融资融券
    """
    def test_margin_trade_stock_total(self):
        """"""
        data = MarginTradingStock(date="2024-10-30", cycle=1, verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_stock_HSA(self):
        """"""
        data = MarginTradingStock(market="沪深A股", date="2024-10-30", cycle=1, verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_margin_trade_stock_KC(self):
        """"""
        data = MarginTradingStock(market="科创板", date="2024-10-30", cycle=1, verbose=True).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestDailyMarginTrade(unittest.TestCase):
    """
    融资融券交易明细:
        日融资融券交易明细
    """

    def test_margin_trade_daily(self):
        """"""
        data = MarginTradingDaily().load()
        print(data.to_markdown(chinese_column=True))



import unittest
from pprint import pprint
from wisecon.stock.financial import *


class TestDividend(unittest.TestCase):

    def test_dividend(self):
        """"""
        data = StockDividend(date="2024-09-30", size=5).load()
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        data = StockDividend(start_date="2022-01-31", security_code="601899", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestStockBalance(unittest.TestCase):
    """"""
    def test_balance(self):
        """"""
        data = StockBalance(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_industry(self):
        """"""
        data = StockBalance(start_date="2024-01-31", size=5, industry_name="纺织服装").load()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        data = StockBalance(start_date="2024-01-31", size=5, security_code="603889").load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestEarnForcast(unittest.TestCase):
    def test_columns(self):
        data = EarnForcast(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = EarnForcast(date="2024-09-30", security_code="301626", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestEarn(unittest.TestCase):
    def test_columns(self):
        data = Earn(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = Earn(date="2024-09-30", security_code="603889", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestFastEarn(unittest.TestCase):
    def test_columns(self):
        data = EarnFast(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = EarnFast(date="2024-09-30", security_code="600995", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestScheduledDisclosure(unittest.TestCase):
    def test_columns(self):
        data = ScheduledDisclosure(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = ScheduledDisclosure(date="2024-09-30", security_code="600816", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestStockIncome(unittest.TestCase):
    def test_columns(self):
        data = StockIncome(date="2024-09-30", size=5).load()
        # pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = StockIncome(date="2024-09-30", security_code="600816", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())


class TestStockCashFlow(unittest.TestCase):
    def test_columns(self):
        data = StockCashFlow(date="2024-09-30", size=5).load()
        pprint(data.to_dict(chinese_column=True)[0], indent=4)
        data.show_columns()
        print(data.to_frame(chinese_column=True).to_markdown())

    def test_stock(self):
        """"""
        data = StockCashFlow(date="2024-09-30", security_code="600816", size=5).load()
        print(data.to_frame(chinese_column=True).to_markdown())



import pandas as pd
from typing import Union


__all__ = [
    "validate_response_data"
]


def validate_response_data(data: Union[list, pd.DataFrame]) -> str:
    """"""
    if len(data) == 0:
        return "No data found."
    prefix = ""
    if len(data) > 50:
        prefix = "Data too large, showing first 50 rows:\n\n"

    if isinstance(data, list):
        data = str(data[:50])
    elif isinstance(data, pd.DataFrame):
        data = data.head(50).to_markdown(index=False)
    data = f"{prefix}{data}"
    return data
